@@ -19,9 +19,10 @@ Naming Conventions
 """
 
 from __future__ import division
-import os
-import sys
-import warnings
+import os, sys, warnings, gzip
+
+import numpy as npy
+
 def _fn_name(): return sys._getframe(1).f_code.co_name
 
 import cairo
@@ -35,25 +36,21 @@ del _version_required
 
 from matplotlib.backend_bases import RendererBase, GraphicsContextBase,\
      FigureManagerBase, FigureCanvasBase
-from matplotlib.cbook      import enumerate, izip
-from matplotlib.figure     import Figure
-from matplotlib.mathtext   import math_parse_s_ft2font
-import matplotlib.numerix as numx
-from matplotlib.transforms import Bbox
-
-
-if hasattr (cairo.ImageSurface, 'create_for_array'):
-   HAVE_CAIRO_NUMPY = True
-else:
-   HAVE_CAIRO_NUMPY = False
-
+from matplotlib.cbook        import enumerate, izip, is_string_like
+from matplotlib.figure       import Figure
+from matplotlib.mathtext     import MathTextParser
+from matplotlib.transforms   import Bbox
+from matplotlib.font_manager import ttfFontProperty
+from matplotlib import rcParams
 
 _debug = False
 #_debug = True
 
-# Image formats that this backend supports - for print_figure()
-IMAGE_FORMAT          = ['eps', 'pdf', 'png', 'ps', 'svg']
-IMAGE_FORMAT_DEFAULT  = 'png'
+# Image::color_conv(format) for draw_image()
+if sys.byteorder == 'little':
+   BYTE_FORMAT = 0 # BGRA
+else:
+   BYTE_FORMAT = 1 # ARGB
 
 
 class RendererCairo(RendererBase):
@@ -91,7 +88,7 @@ class RendererCairo(RendererBase):
         self.dpi = dpi
         self.text_ctx = cairo.Context (
            cairo.ImageSurface (cairo.FORMAT_ARGB32,1,1))
-
+        self.mathtext_parser = MathTextParser('Cairo')
 
     def set_ctx_from_surface (self, surface):
        self.ctx = cairo.Context (surface)
@@ -128,24 +125,19 @@ class RendererCairo(RendererBase):
         #_.ctx.restore() # revert to the default attributes
 
 
-    def draw_arc(self, gc, rgbFace, x, y, width, height, angle1, angle2, rotation):
+    def draw_arc(self, gc, rgbFace, x, y, width, height, angle1, angle2,
+                 rotation):
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
-        # draws circular arcs where width=height
-        # FIXME
-        # to get a proper arc of width/height you can use translate() and
-        # scale(), see draw_arc() manual page
-        
-        #radius = (height + width) / 4
         ctx = gc.ctx
         ctx.save()
+        ctx.translate(x, self.height - y)
         ctx.rotate(rotation)
         ctx.scale(width / 2.0, height / 2.0)
-        ctx.arc(0.0, 0.0, 1.0, 0.0, 2*numx.pi)
+        ctx.new_sub_path()
+        ctx.arc(0.0, 0.0, 1.0, npy.pi * angle1 / 180.,
+                npy.pi * angle2 / 180.)
         ctx.restore()
-        
-        #ctx.new_path()
-        #ctx.arc (x, self.height - y, radius,
-        #         angle1 * numx.pi/180.0, angle2 * numx.pi/180.0)
+
         self._fill_and_stroke (ctx, rgbFace)
 
 
@@ -153,25 +145,14 @@ class RendererCairo(RendererBase):
         # bbox - not currently used
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
 
-        if numx.which[0] == "numarray":
-            warnings.warn("draw_image() currently works for numpy, but not "
-                          "numarray")
-            return
-
-        if not HAVE_CAIRO_NUMPY:
-            warnings.warn("cairo with Numeric support is required for "
-                          "draw_image()")
-            return
-
         im.flipud_out()
 
-        rows, cols, buf = im.buffer_argb32()  # ARGB32, but colors still wrong
-        X = numx.fromstring (buf, numx.UInt8)
-        X.shape = rows, cols, 4
-
+        rows, cols, buf = im.color_conv (BYTE_FORMAT)
+        surface = cairo.ImageSurface.create_for_data (
+                      buf, cairo.FORMAT_ARGB32, cols, rows, cols*4)
         # function does not pass a 'gc' so use renderer.ctx
         ctx = self.ctx
-        surface = cairo.ImageSurface.create_for_array (X)
+        y = self.height - y - rows
         ctx.set_source_surface (surface, x, y)
         ctx.paint()
 
@@ -259,7 +240,7 @@ class RendererCairo(RendererBase):
         # render by drawing a 0.5 radius circle
         ctx = gc.ctx
         ctx.new_path()
-        ctx.arc (x, self.height - y, 0.5, 0, 2*numx.pi)
+        ctx.arc (x, self.height - y, 0.5, 0, 2*npy.pi)
         self._fill_and_stroke (ctx, gc.get_rgb())
 
 
@@ -310,67 +291,48 @@ class RendererCairo(RendererBase):
 
            ctx.save()
            if angle:
-              ctx.rotate (-angle * numx.pi / 180)
+              ctx.rotate (-angle * npy.pi / 180)
            ctx.set_font_size (size)
-           ctx.show_text (s)
+           ctx.show_text (s.encode("utf-8"))
            ctx.restore()
 
-
     def _draw_mathtext(self, gc, x, y, s, prop, angle):
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
-       # mathtext using the gtk/gdk method
 
-        if numx.which[0] == "numarray":
-            warnings.warn("_draw_mathtext() currently works for numpy, but "
-                          "not numarray")
-            return
+        ctx = gc.ctx
+        width, height, descent, glyphs, rects = self.mathtext_parser.parse(
+            s, self.dpi.get(), prop)
 
-        if not HAVE_CAIRO_NUMPY:
-            warnings.warn("cairo with Numeric support is required for "
-                          "_draw_mathtext()")
-            return
+        ctx.save()
+        ctx.translate(x, y)
+        if angle:
+           ctx.rotate (-angle * npy.pi / 180)
+           
+        for font, fontsize, s, ox, oy in glyphs:
+           ctx.new_path()
+           ctx.move_to(ox, oy)
+           
+           fontProp = ttfFontProperty(font)
+           ctx.save()
+           ctx.select_font_face (fontProp.name,
+                                 self.fontangles [fontProp.style],
+                                 self.fontweights[fontProp.weight])
 
-        size = prop.get_size_in_points()
-        width, height, fonts = math_parse_s_ft2font(
-            s, self.dpi.get(), size)
+           # size = prop.get_size_in_points() * self.dpi.get() / 96.0
+           size = fontsize * self.dpi.get() / 72.0
+           ctx.set_font_size(size)
+           ctx.show_text(s.encode("utf-8"))
+           ctx.restore()
 
-        if angle==90:
-            width, height = height, width
-            x -= width
-        y -= height
-
-        imw, imh, s = fonts[0].image_as_str()
-        N = imw*imh
-
-        # a numpixels by num fonts array
-        Xall = numx.zeros((N,len(fonts)), typecode=numx.UInt8)
-
-        for i, font in enumerate(fonts):
-            if angle == 90:
-                font.horiz_image_to_vert_image() # <-- Rotate
-            imw, imh, s = font.image_as_str()
-            Xall[:,i] = numx.fromstring(s, numx.UInt8)
-
-        # get the max alpha at each pixel
-        Xs = numx.mlab.max (Xall,1)
-
-        # convert it to it's proper shape
-        Xs.shape = imh, imw
-
-        pa = numx.zeros(shape=(imh,imw,4), typecode=numx.UInt8)
-        rgb = gc.get_rgb()
-        pa[:,:,0] = int(rgb[0]*255)
-        pa[:,:,1] = int(rgb[1]*255)
-        pa[:,:,2] = int(rgb[2]*255)
-        pa[:,:,3] = Xs
-
-        # works for numpy pa, not a numarray pa
-        surface = cairo.ImageSurface.create_for_array (pa)
-        gc.ctx.set_source_surface (surface, x, y)
-        gc.ctx.paint()
-        #gc.ctx.show_surface (surface, imw, imh)
+        for ox, oy, w, h in rects:
+           ctx.new_path()
+           ctx.rectangle (ox, oy, w, h)
+           ctx.set_source_rgb (0, 0, 0)
+           ctx.fill_preserve()
 
+        ctx.restore()
 
+        
     def flipy(self):
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
         return True
@@ -383,12 +345,12 @@ class RendererCairo(RendererBase):
         return self.width, self.height
 
 
-    def get_text_width_height(self, s, prop, ismath):
+    def get_text_width_height_descent(self, s, prop, ismath):
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
         if ismath:
-            width, height, fonts = math_parse_s_ft2font(
-               s, self.dpi.get(), prop.get_size_in_points())
-            return width, height
+            width, height, descent, fonts, used_characters = self.mathtext_parser.parse(
+               s, self.dpi.get(), prop)
+            return width, height, descent
 
         ctx = self.text_ctx
         ctx.save()
@@ -407,10 +369,10 @@ class RendererCairo(RendererBase):
         # save/restore prevents the problem
         ctx.set_font_size (size)
 
-        w, h = ctx.text_extents (s)[2:4]
+        y_bearing, w, h = ctx.text_extents (s)[1:4]
         ctx.restore()
 
-        return w, h
+        return w, h, h + y_bearing
 
 
     def new_gc(self):
@@ -472,11 +434,9 @@ class GraphicsContextCairo(GraphicsContextBase):
         ctx = self.ctx
         ctx.new_path()
         ctx.rectangle (x, self.renderer.height - h - y, w, h)
-
-        # enabline ctx.clip() causes problems:
-        # line_styles.py - only see first axes
-        # simple_plot.py - lose text
-        #ctx.clip ()
+        ctx.clip ()
+        # Alternative: just set _cliprect here and actually set cairo clip rect
+        # in fill_and_stroke() inside ctx.save() ... ctx.restore()
 
 
     def set_dashes(self, offset, dashes):
@@ -485,7 +445,7 @@ class GraphicsContextCairo(GraphicsContextBase):
             self.ctx.set_dash([], 0)  # switch dashes off
         else:
             self.ctx.set_dash (
-               self.renderer.points_to_pixels (numx.asarray(dashes)), offset)
+               self.renderer.points_to_pixels (npy.asarray(dashes)), offset)
 
 
     def set_foreground(self, fg, isRGB=None):
@@ -524,37 +484,7 @@ def new_figure_manager(num, *args, **kwargs): # called by backends/__init__.py
 
 
 class FigureCanvasCairo (FigureCanvasBase):
-    def print_figure(self, fo, dpi=150, facecolor='w', edgecolor='w',
-                     orientation='portrait', format=None, **kwargs):
-        if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
-        # settings for printing
-        self.figure.dpi.set(dpi)
-        self.figure.set_facecolor(facecolor)
-        self.figure.set_edgecolor(edgecolor)
-
-        if format is None and isinstance (fo, basestring):
-            # get format from filename extension
-            format = os.path.splitext(fo)[1][1:]
-
-        if format is not None:
-            format = format.lower()
-
-        if format == 'png':
-            self._save_png (fo)
-        elif format in ('pdf', 'ps', 'svg'):
-            self._save (fo, format, orientation, **kwargs)
-        elif format == 'eps': # backend_ps for eps
-            from backend_ps import FigureCanvasPS  as FigureCanvas
-            fc = FigureCanvas(self.figure)
-            fc.print_figure (filename, dpi, facecolor, edgecolor,
-                             orientation, **kwargs)
-        else:
-            warnings.warn('Format "%s" is not supported.\n'
-                          'Supported formats: '
-                          '%s.' % (format, ', '.join(IMAGE_FORMAT)))
-
-
-    def _save_png (self, fobj):
+    def print_png(self, fobj, *args, **kwargs):
         width, height = self.get_width_height()
 
         renderer = RendererCairo (self.figure.dpi)
@@ -564,9 +494,23 @@ class FigureCanvasCairo (FigureCanvasBase):
 
         self.figure.draw (renderer)
         surface.write_to_png (fobj)
+    
+    def print_pdf(self, fobj, *args, **kwargs):
+        return self._save(fobj, 'pdf', *args, **kwargs)
 
+    def print_ps(self, fobj, *args, **kwargs):
+        return self._save(fobj, 'ps', *args, **kwargs)
 
-    def _save (self, fo, format, orientation, **kwargs):
+    def print_svg(self, fobj, *args, **kwargs):
+        return self._save(fobj, 'svg', *args, **kwargs)
+
+    def print_svgz(self, fobj, *args, **kwargs):
+        return self._save(fobj, 'svgz', *args, **kwargs)
+    
+    def get_default_filetype(self):
+        return rcParams['cairo.format']
+    
+    def _save (self, fo, format, **kwargs):
         # save PDF/PS/SVG
         orientation = kwargs.get('orientation', 'portrait')
 
@@ -589,10 +533,15 @@ class FigureCanvasCairo (FigureCanvasBase):
                 raise RuntimeError ('cairo has not been compiled with PDF '
                                     'support enabled')
             surface = cairo.PDFSurface (fo, width_in_points, height_in_points)
-        elif format == 'svg':
+        elif format in ('svg', 'svgz'):
             if not cairo.HAS_SVG_SURFACE:
                 raise RuntimeError ('cairo has not been compiled with SVG '
                                     'support enabled')
+            if format == 'svgz':
+                filename = fo
+                if is_string_like(fo):
+                    fo = open(fo, 'wb')
+                fo = gzip.GzipFile(None, 'wb', fileobj=fo)
             surface = cairo.SVGSurface (fo, width_in_points, height_in_points)
         else:
            warnings.warn ("unknown format: %s" % format)
@@ -605,7 +554,7 @@ class FigureCanvasCairo (FigureCanvasBase):
         ctx = renderer.ctx
 
         if orientation == 'landscape':
-            ctx.rotate (numx.pi/2)
+            ctx.rotate (npy.pi/2)
             ctx.translate (0, -height_in_points)
             # cairo/src/cairo_ps_surface.c
             # '%%Orientation: Portrait' is always written to the file header
