@@ -2,10 +2,13 @@
 Figure class -- add docstring here!
 """
 import sys
+
+import numpy as npy
+
 import artist
 from artist import Artist
 from axes import Axes, Subplot, PolarSubplot, PolarAxes
-from cbook import flatten, allequal, popd, Stack, iterable, dedent
+from cbook import flatten, allequal, Stack, iterable, dedent
 import _image
 import colorbar as cbar
 from colors import Normalize, rgb2hex
@@ -16,8 +19,6 @@ from text import Text, _process_text_args
 
 from legend import Legend
 from transforms import Bbox, Value, Point, get_bbox_transform, unit_bbox
-from numerix import array, clip, transpose, minimum, maximum
-from mlab import linspace, meshgrid
 from ticker import FormatStrFormatter
 from cm import ScalarMappable
 from contour import ContourSet
@@ -102,6 +103,9 @@ class SubplotParams:
 
 class Figure(Artist):
 
+    def __str__(self):
+        return "Figure(%gx%g)"%(self.figwidth.get(),self.figheight.get())
+
     def __init__(self,
                  figsize   = None,  # defaults to rc figure.figsize
                  dpi       = None,  # defaults to rc figure.dpi
@@ -117,10 +121,6 @@ class Figure(Artist):
         subplotpars is a SubplotParams instance, defaults to rc
         """
         Artist.__init__(self)
-        #self.set_figure(self)
-        self._axstack = Stack()  # maintain the current axes
-        self._axobservers = []
-        self._seen = {}          # axes args we've seen
 
         if figsize is None  : figsize   = rcParams['figure.figsize']
         if dpi is None      : dpi       = rcParams['figure.dpi']
@@ -156,10 +156,39 @@ class Figure(Artist):
 
         self.subplotpars = subplotpars
 
+        self._axstack = Stack()  # maintain the current axes
+        self.axes = []
         self.clf()
 
         self._cachedRenderer = None
 
+    def autofmt_xdate(self, bottom=0.2, rotation=30, ha='right'):
+        """
+        A common use case is a number of subplots with shared xaxes
+        where the x-axis is date data.  The ticklabels are often
+        long,and it helps to rotate them on the bottom subplot and
+        turn them off on other subplots.  This function will raise a
+        RuntimeError if any of the Axes are not Subplots.
+
+        bottom : the bottom of the subplots for subplots_adjust
+        rotation: the rotation of the xtick labels
+        ha : the horizontal alignment of the xticklabels
+
+
+        """
+
+        for ax in self.get_axes():
+            if not hasattr(ax, 'is_last_row'):
+                raise RuntimeError('Axes must be subplot instances; found %s'%type(ax))
+            if ax.is_last_row():
+                for label in ax.get_xticklabels():
+                    label.set_ha(ha)
+                    label.set_rotation(rotation)
+            else:
+                for label in ax.get_xticklabels():
+                    label.set_visible(False)
+        self.subplots_adjust(bottom=bottom)
+
     def get_children(self):
         'get a list of artists contained in the figure'
         children = [self.figurePatch]
@@ -170,11 +199,18 @@ class Figure(Artist):
         children.extend(self.images)
         children.extend(self.legends)
         return children
-    
-    def pick(self, mouseevent):
-        for a in self.get_children():
-            a.pick(mouseevent)
-    
+
+    def contains(self, mouseevent):
+        """Test whether the mouse event occurred on the figure.
+
+        Returns True,{}
+        """
+        if callable(self._contains): return self._contains(self,mouseevent)
+        #inside = mouseevent.x >= 0 and mouseevent.y >= 0
+        inside = self.bbox.contains(mouseevent.x,mouseevent.y)
+
+        return inside,{}
+
     def get_window_extent(self, *args, **kwargs):
         'get the figure bounding box in display space; kwargs are void'
         return self.bbox
@@ -451,7 +487,7 @@ class Figure(Artist):
             assert(a.get_figure() is self)
         else:
             rect = args[0]
-            ispolar = popd(kwargs, 'polar', False)
+            ispolar = kwargs.pop('polar', False)
 
             if ispolar:
                 a = PolarAxes(self, rect, **kwargs)
@@ -499,7 +535,7 @@ class Figure(Artist):
             a = args[0]
             assert(a.get_figure() is self)
         else:
-            ispolar = popd(kwargs, 'polar', False)
+            ispolar = kwargs.pop('polar', False)
             if ispolar:
                 a = PolarSubplot(self, *args, **kwargs)
             else:
@@ -517,7 +553,13 @@ class Figure(Artist):
         """
         Clear the figure
         """
-        self.axes = []
+        for ax in tuple(self.axes):  # Iterate over the copy.
+            ax.cla()
+            self.delaxes(ax)         # removes ax from self.axes
+
+        toolbar = getattr(self.canvas, 'toolbar', None)
+        if toolbar is not None:
+            toolbar.update()
         self._axstack.clear()
         self._seen = {}
         self.lines = []
@@ -525,6 +567,7 @@ class Figure(Artist):
         self.texts=[]
         self.images = []
         self.legends = []
+        self._axobservers = []
 
     def clear(self):
         """
@@ -589,7 +632,7 @@ class Figure(Artist):
     def get_axes(self):
         return self.axes
 
-    def legend(self, handles, labels, loc, **kwargs):
+    def legend(self, handles, labels, *args, **kwargs):
         """
         Place a legend in the figure.  Labels are a sequence of
         strings, handles is a sequence of line or patch instances, and
@@ -603,8 +646,8 @@ class Figure(Artist):
 
         The LOC location codes are
 
-          'best' : 0,          (currently not supported, defaults to upper right)
-          'upper right'  : 1,  (default)
+          'best' : 0,          (currently not supported for figure legends)
+          'upper right'  : 1,
           'upper left'   : 2,
           'lower left'   : 3,
           'lower right'  : 4,
@@ -622,7 +665,7 @@ class Figure(Artist):
 
         The legend instance is returned.  The following kwargs are supported:
 
-        isaxes=True           # whether this is an axes legend
+        loc = "upper right" #
         numpoints = 4         # the number of points in the legend line
         prop = FontProperties(size='smaller')  # the font property
         pad = 0.2             # the fractional whitespace inside the legend border
@@ -637,7 +680,7 @@ class Figure(Artist):
 
 
         handles = flatten(handles)
-        l = Legend(self, handles, labels, loc, isaxes=False, **kwargs)
+        l = Legend(self, handles, labels, *args, **kwargs)
         self._set_artist_props(l)
         self.legends.append(l)
         return l
@@ -702,9 +745,11 @@ class Figure(Artist):
                 used.  and are deduced by the extension to fname.
                 Possibilities are eps, jpeg, pdf, png, ps, svg.  fname
                 can also be a file or file-like object - cairo backend
-                only.  dpi - is the resolution in dots per inch.  If
-                None it will default to the value savefig.dpi in the
-                matplotlibrc file
+                only.
+
+        dpi - is the resolution in dots per inch.  If
+              None it will default to the value savefig.dpi in the
+              matplotlibrc file
 
         facecolor and edgecolor are the colors of the figure rectangle
 
@@ -715,9 +760,7 @@ class Figure(Artist):
         through 'a10', or 'b0' through 'b10' - only supported for postscript
         output
 
-        format - one of 'pdf', 'png', 'ps', 'svg'. It is used to specify the
-                 output when fname is a file or file-like object - cairo
-                 backend only.
+        format - one of the file extensions supported by the active backend.
         """
 
         for key in ('dpi', 'facecolor', 'edgecolor'):
@@ -726,18 +769,9 @@ class Figure(Artist):
 
         self.canvas.print_figure(*args, **kwargs)
 
-    def colorbar(self, mappable, cax=None, **kw):
-        # Temporary compatibility code:
-        old = ('tickfmt', 'cspacing', 'clabels', 'edgewidth', 'edgecolor')
-        oldkw = [k for k in old if kw.has_key(k)]
-        if oldkw:
-            msg = 'Old colorbar kwargs (%s) found; using colorbar_classic.' % (','.join(oldkw),)
-            warnings.warn(msg, DeprecationWarning)
-            self.colorbar_classic(mappable, cax, **kw)
-            return cax
-        # End of compatibility code block.
-        orientation = kw.get('orientation', 'vertical')
-        ax = self.gca()
+    def colorbar(self, mappable, cax=None, ax=None, **kw):
+        if ax is None:
+            ax = self.gca()
         if cax is None:
             cax, kw = cbar.make_axes(ax, **kw)
         cb = cbar.Colorbar(cax, mappable, **kw)
@@ -751,170 +785,6 @@ class Figure(Artist):
         Documentation for the pylab thin wrapper: %s
         '''% cbar.colorbar_doc
 
-    def colorbar_classic(self, mappable,  cax=None,
-                    orientation='vertical', tickfmt='%1.1f',
-                    cspacing='proportional',
-                    clabels=None, drawedges=False, edgewidth=0.5,
-                    edgecolor='k'):
-        """
-        Create a colorbar for mappable image
-
-        mappable is the cm.ScalarMappable instance that you want the
-        colorbar to apply to, e.g. an Image as returned by imshow or a
-        PatchCollection as returned by scatter or pcolor.
-
-        tickfmt is a format string to format the colorbar ticks
-
-        cax is a colorbar axes instance in which the colorbar will be
-        placed.  If None, as default axesd will be created resizing the
-        current aqxes to make room for it.  If not None, the supplied axes
-        will be used and the other axes positions will be unchanged.
-
-        orientation is the colorbar orientation: one of 'vertical' | 'horizontal'
-
-        cspacing controls how colors are distributed on the colorbar.
-        if cspacing == 'linear', each color occupies an equal area
-        on the colorbar, regardless of the contour spacing.
-        if cspacing == 'proportional' (Default), the area each color
-        occupies on the the colorbar is proportional to the contour interval.
-        Only relevant for a Contour image.
-
-        clabels can be a sequence containing the
-        contour levels to be labelled on the colorbar, or None (Default).
-        If clabels is None, labels for all contour intervals are
-        displayed. Only relevant for a Contour image.
-
-        if drawedges == True, lines are drawn at the edges between
-        each color on the colorbar. Default False.
-
-        edgecolor is the line color delimiting the edges of the colors
-        on the colorbar (if drawedges == True). Default black ('k')
-
-        edgewidth is the width of the lines delimiting the edges of
-        the colors on the colorbar (if drawedges == True). Default 0.5
-
-        return value is the colorbar axes instance
-        """
-
-        if orientation not in ('horizontal', 'vertical'):
-            raise ValueError('Orientation must be horizontal or vertical')
-
-        if isinstance(mappable, FigureImage) and cax is None:
-            raise TypeError('Colorbars for figure images currently not supported unless you provide a colorbar axes in cax')
-
-
-        ax = self.gca()
-
-        cmap = mappable.cmap
-
-        if cax is None:
-            l,b,w,h = ax.get_position()
-            if orientation=='vertical':
-                neww = 0.8*w
-                ax.set_position((l,b,neww,h), 'both')
-                cax = self.add_axes([l + 0.9*w, b, 0.1*w, h])
-            else:
-                newh = 0.8*h
-                ax.set_position((l,b+0.2*h,w,newh), 'both')
-                cax = self.add_axes([l, b, w, 0.1*h])
-
-        else:
-            if not isinstance(cax, Axes):
-                raise TypeError('Expected an Axes instance for cax')
-
-        norm = mappable.norm
-        if norm.vmin is None or norm.vmax is None:
-            mappable.autoscale()
-        cmin = norm.vmin
-        cmax = norm.vmax
-        if isinstance(mappable, ContourSet):
-        # mappable image is from contour or contourf
-            clevs = mappable.levels
-            clevs = minimum(clevs, cmax)
-            clevs = maximum(clevs, cmin)
-            isContourSet = True
-        elif isinstance(mappable, ScalarMappable):
-        # from imshow or pcolor.
-            isContourSet = False
-            clevs = linspace(cmin, cmax, cmap.N+1) # boundaries, hence N+1
-        else:
-            raise TypeError("don't know how to handle type %s"%type(mappable))
-
-        N = len(clevs)
-        C = array([clevs, clevs])
-        if cspacing == 'linear':
-            X, Y = meshgrid(clevs, [0, 1])
-        elif cspacing == 'proportional':
-            X, Y = meshgrid(linspace(cmin, cmax, N), [0, 1])
-        else:
-            raise ValueError("cspacing must be 'linear' or 'proportional'")
-
-        if orientation=='vertical':
-            args = (transpose(Y), transpose(C), transpose(X), clevs)
-        else:
-            args = (C, Y, X, clevs)
-        #If colors were listed in the original mappable, then
-        # let contour handle them the same way.
-        colors = getattr(mappable, 'colors', None)
-        if colors is not None:
-            kw = {'colors': colors}
-        else:
-            kw = {'cmap':cmap, 'norm':norm}
-        if isContourSet and not mappable.filled:
-            CS = cax.contour(*args, **kw)
-            colls = mappable.collections
-            for ii in range(len(colls)):
-                CS.collections[ii].set_linewidth(colls[ii].get_linewidth())
-        else:
-            kw['antialiased'] = False
-            CS = cax.contourf(*args, **kw)
-        if drawedges:
-            for col in CS.collections:
-                col.set_edgecolor(edgecolor)
-                col.set_linewidth(edgewidth)
-
-        mappable.add_observer(CS)
-        mappable.set_colorbar(CS, cax)
-
-
-        if isContourSet:
-            if cspacing == 'linear':
-                ticks = linspace(cmin, cmax, N)
-            else:
-                ticks = clevs
-            if cmin == mappable.levels[0]:
-                ticklevs = clevs
-            else: # We are not showing the full ends of the range.
-                ticks = ticks[1:-1]
-                ticklevs = clevs[1:-1]
-            labs = [tickfmt % lev for lev in ticklevs]
-            if clabels is not None:
-                for i, lev in enumerate(ticklevs):
-                    if lev not in clabels:
-                        labs[i] = ''
-
-
-        if orientation=='vertical':
-            cax.set_xticks([])
-            cax.yaxis.tick_right()
-            cax.yaxis.set_label_position('right')
-            if isContourSet:
-                cax.set_yticks(ticks)
-                cax.set_yticklabels(labs)
-            else:
-                cax.yaxis.set_major_formatter(FormatStrFormatter(tickfmt))
-        else:
-            cax.set_yticks([])
-            if isContourSet:
-                cax.set_xticks(ticks)
-                cax.set_xticklabels(labs)
-            else:
-                cax.xaxis.set_major_formatter(FormatStrFormatter(tickfmt))
-
-        self.sca(ax)
-        return cax
-
-
     def subplots_adjust(self, *args, **kwargs):
         """
         subplots_adjust(self, left=None, bottom=None, right=None, top=None,
@@ -973,8 +843,8 @@ def figaspect(arg):
 
     # min/max sizes to respect when autoscaling.  If John likes the idea, they
     # could become rc parameters, for now they're hardwired.
-    figsize_min = array((4.0,2.0)) # min length for width/height
-    figsize_max = array((16.0,16.0)) # max length for width/height
+    figsize_min = npy.array((4.0,2.0)) # min length for width/height
+    figsize_max = npy.array((16.0,16.0)) # max length for width/height
     #figsize_min = rcParams['figure.figsize_min']
     #figsize_max = rcParams['figure.figsize_max']
 
@@ -989,7 +859,7 @@ def figaspect(arg):
     fig_height = rcParams['figure.figsize'][1]
 
     # New size for the figure, keeping the aspect ratio of the caller
-    newsize = array((fig_height/arr_ratio,fig_height))
+    newsize = npy.array((fig_height/arr_ratio,fig_height))
 
     # Sanity checks, don't drop either dimension below figsize_min
     newsize /= min(1.0,*(newsize/figsize_min))
@@ -999,7 +869,7 @@ def figaspect(arg):
 
     # Finally, if we have a really funky aspect ratio, break it but respect
     # the min/max dimensions (we don't want figures 10 feet tall!)
-    newsize = clip(newsize,figsize_min,figsize_max)
+    newsize = npy.clip(newsize,figsize_min,figsize_max)
     return newsize
 
 artist.kwdocd['Figure'] = artist.kwdoc(Figure)
