@@ -1,19 +1,42 @@
 from __future__ import division
 import math
-from matplotlib import rcParams
-from numerix import array, arange, sin, cos, pi, Float, sqrt, \
-     matrixmultiply, sqrt, nonzero, equal, asarray, dot, concatenate
-from artist import Artist, setp
-from cbook import enumerate, popd
-from colors import colorConverter
-from lines import Line2D
-from transforms import bound_vertices
 
-from numerix.mlab import amin
-from mlab import dist_point_to_segment
+import matplotlib as mpl
+import numpy as npy
+import matplotlib.cbook as cbook
+import matplotlib.artist as artist
+import matplotlib.colors as colors
+import matplotlib.lines as lines
+import matplotlib.transforms as transforms
+import matplotlib.nxutils as nxutils
+import matplotlib.mlab as mlab
+import matplotlib.artist as artist
+from matplotlib import transforms as mtrans
+import agg
 
+# these are not available for the object inspector until after the
+# class is build so we define an initial set here for the init
+# function and they will be overridden after object defn
+artist.kwdocd['Patch'] = """\
+          alpha: float
+          animated: [True | False]
+          antialiased or aa: [True | False]
+          clip_box: a matplotlib.transform.Bbox instance
+          clip_on: [True | False]
+          edgecolor or ec: any matplotlib color
+          facecolor or fc: any matplotlib color
+          figure: a matplotlib.figure.Figure instance
+          fill: [True | False]
+          hatch: unknown
+          label: any string
+          linewidth or lw: float
+          lod: [True | False]
+          transform: a matplotlib.transform transformation instance
+          visible: [True | False]
+          zorder: any number
+          """
 
-class Patch(Artist):
+class Patch(artist.Artist):
     """
     A patch is a 2D thingy with a face color and an edge color
 
@@ -22,6 +45,9 @@ class Patch(Artist):
 
     """
     zorder = 1
+    def __str__(self):
+        return str(self.__class__).split('.')[-1]
+
     def __init__(self,
                  edgecolor=None,
                  facecolor=None,
@@ -31,12 +57,16 @@ class Patch(Artist):
                  fill=1,
                  **kwargs
                  ):
-        Artist.__init__(self)
+        """
+        The following kwarg properties are supported
+        %(Patch)s
+        """
+        artist.Artist.__init__(self)
 
-        if edgecolor is None: edgecolor = rcParams['patch.edgecolor']
-        if facecolor is None: facecolor = rcParams['patch.facecolor']
-        if linewidth is None: linewidth = rcParams['patch.linewidth']
-        if antialiased is None: antialiased = rcParams['patch.antialiased']
+        if edgecolor is None: edgecolor = mpl.rcParams['patch.edgecolor']
+        if facecolor is None: facecolor = mpl.rcParams['patch.facecolor']
+        if linewidth is None: linewidth = mpl.rcParams['patch.linewidth']
+        if antialiased is None: antialiased = mpl.rcParams['patch.antialiased']
 
         self._edgecolor = edgecolor
         self._facecolor = facecolor
@@ -45,10 +75,30 @@ class Patch(Artist):
         self._hatch = hatch
         self.fill = fill
 
-        if len(kwargs): setp(self, **kwargs)
+        if len(kwargs): artist.setp(self, **kwargs)
+    __init__.__doc__ = cbook.dedent(__init__.__doc__) % artist.kwdocd
+
+
+
+    def contains(self, mouseevent):
+        """Test whether the mouse event occurred in the patch.
+
+        Returns T/F, {}
+        """
+        if callable(self._contains): return self._contains(self,mouseevent)
+
+        try:
+            # TODO: make this consistent with patch collection algorithm
+            x, y = self.get_transform().inverse_xy_tup((mouseevent.x, mouseevent.y))
+            xyverts = self.get_verts()
+            inside = nxutils.pnpoly(x, y, xyverts)
+            #print str(self),"%g,%g is in"%(x,y),xyverts,inside
+            return inside,{}
+        except ValueError:
+            return False,{}
 
     def update_from(self, other):
-        Artist.update_from(self, other)
+        artist.Artist.update_from(self, other)
         self.set_edgecolor(other.get_edgecolor())
         self.set_facecolor(other.get_facecolor())
         self.set_fill(other.get_fill())
@@ -58,6 +108,7 @@ class Patch(Artist):
         self.set_figure(other.get_figure())
         self.set_alpha(other.get_alpha())
 
+
     def get_antialiased(self):
         return self._antialiased
 
@@ -82,7 +133,7 @@ class Patch(Artist):
         """
         Set the patch edge color
 
-        ACCEPTS: any matplotlib color - see help(colors)
+        ACCEPTS: any matplotlib color
         """
         self._edgecolor = color
 
@@ -90,7 +141,7 @@ class Patch(Artist):
         """
         Set the patch face color
 
-        ACCEPTS: any matplotlib color - see help(colors)
+        ACCEPTS: any matplotlib color
         """
         self._facecolor = color
 
@@ -147,22 +198,29 @@ class Patch(Artist):
         if not self.get_visible(): return
         #renderer.open_group('patch')
         gc = renderer.new_gc()
-        gc.set_foreground(self._edgecolor)
-        gc.set_linewidth(self._linewidth)
+
+        if cbook.is_string_like(self._edgecolor) and self._edgecolor.lower()=='none':
+            gc.set_linewidth(0)
+        else:        
+            gc.set_foreground(self._edgecolor)
+            gc.set_linewidth(self._linewidth)
+
         gc.set_alpha(self._alpha)
         gc.set_antialiased(self._antialiased)
-        if self.get_clip_on(): gc.set_clip_rectangle(
-            self.clipbox.get_bounds())
+        self._set_gc_clip(gc)
         gc.set_capstyle('projecting')
 
-        if not self.fill or self._facecolor is None: rgbFace = None
-        else: rgbFace = colorConverter.to_rgb(self._facecolor)
+        if (not self.fill or self._facecolor is None or
+            (cbook.is_string_like(self._facecolor) and self._facecolor.lower()=='none')):
+            rgbFace = None
+        else:
+            rgbFace = colors.colorConverter.to_rgb(self._facecolor)
 
         if self._hatch:
             gc.set_hatch(self._hatch )
 
         verts = self.get_verts()
-        tverts = self._transform.seq_xy_tups(verts)
+        tverts = self.get_transform().seq_xy_tups(verts)
 
         renderer.draw_polygon(gc, rgbFace, tverts)
 
@@ -175,10 +233,11 @@ class Patch(Artist):
         """
         raise NotImplementedError('Derived must override')
 
+
     def get_window_extent(self, renderer=None):
         verts = self.get_verts()
-        tverts = self._transform.seq_xy_tups(verts)
-        return bound_vertices(tverts)
+        tverts = self.get_transform().seq_xy_tups(verts)
+        return transforms.bound_vertices(tverts)
 
 
 
@@ -216,25 +275,34 @@ class Patch(Artist):
         'alias for get_facecolor'
         return self.get_facecolor()
 
+
 class Shadow(Patch):
-    def __init__(self, patch, ox, oy, props=None):
+    def __str__(self):
+        return "Shadow(%s)"%(str(self.patch))
+
+    def __init__(self, patch, ox, oy, props=None, **kwargs):
         """
         Create a shadow of the patch offset by ox, oy.  props, if not None is
         a patch property update dictionary.  If None, the shadow will have
         have the same color as the face, but darkened
+
+        kwargs are
+        %(Patch)s
         """
         Patch.__init__(self)
         self.ox, self.oy = ox, oy
         self.patch = patch
         self.props = props
         self._update()
+    __init__.__doc__ = cbook.dedent(__init__.__doc__) % artist.kwdocd
+
 
     def _update(self):
         self.update_from(self.patch)
         if self.props is not None:
             self.update(self.props)
         else:
-            r,g,b,a = colorConverter.to_rgba(self.patch.get_facecolor())
+            r,g,b,a = colors.colorConverter.to_rgba(self.patch.get_facecolor())
             rho = 0.3
             r = rho*r
             g = rho*g
@@ -245,8 +313,8 @@ class Shadow(Patch):
 
     def get_verts(self):
         verts = self.patch.get_verts()
-        xs = [x+self.ox for x,y in verts]
-        ys = [y+self.oy for x,y in verts]
+        xs = self.convert_xunits([x+self.ox for x,y in verts])
+        ys = self.convert_yunits([y+self.oy for x,y in verts])
         return zip(xs, ys)
 
     def _draw(self, renderer):
@@ -261,6 +329,10 @@ class Rectangle(Patch):
 
     """
 
+    def __str__(self):
+        return str(self.__class__).split('.')[-1] \
+            + "(%g,%g;%gx%g)"%(self.xy[0],self.xy[1],self.width,self.height)
+
     def __init__(self, xy, width, height,
                  **kwargs):
         """
@@ -270,20 +342,29 @@ class Rectangle(Patch):
 
         fill is a boolean indicating whether to fill the rectangle
 
+        Valid kwargs are:
+        %(Patch)s
         """
 
         Patch.__init__(self, **kwargs)
 
-        self.xy  = array(xy, Float)
+        self.xy  = list(xy)
         self.width, self.height = width, height
+    __init__.__doc__ = cbook.dedent(__init__.__doc__) % artist.kwdocd
+
 
     def get_verts(self):
         """
         Return the vertices of the rectangle
         """
         x, y = self.xy
-        return ( (x, y), (x, y+self.height),
-                 (x+self.width, y+self.height), (x+self.width, y),
+        left = self.convert_xunits(x)
+        right = self.convert_xunits(x + self.width)
+        bottom = self.convert_yunits(y)
+        top = self.convert_yunits(y+self.height)
+
+        return ( (left, bottom), (left, top),
+                 (right, top), (right, bottom),
                  )
 
     def get_x(self):
@@ -344,104 +425,158 @@ class Rectangle(Patch):
             l,b,w,h = args[0]
         else:
             l,b,w,h = args
-        self.xy = array([float(l),float(b)])
+        self.xy = [l,b]
         self.width = w
         self.height = h
 
 
 class RegularPolygon(Patch):
     """
-    A regular polygon patch.  xy is a length 2 tuple (the center)
-    numVertices is the number of vertices.  Radius is the distance
-    from the center to each of the vertices.  Orientation is in
-    radians and rotates the polygon.
-
-
+    A regular polygon patch.
     """
+    def __str__(self):
+        return "Poly%d(%g,%g)"%(self.numVertices,self.xy[0],self.xy[1])
+
     def __init__(self, xy, numVertices, radius=5, orientation=0,
                  **kwargs):
+        """
+        xy is a length 2 tuple (the center)
+        numVertices is the number of vertices.
+        radius is the distance from the center to each of the vertices.
+        orientation is in radians and rotates the polygon.
 
+        Valid kwargs are:
+        %(Patch)s
+        """
         Patch.__init__(self, **kwargs)
 
-        self.xy = xy
+        self.xy = list(xy)
         self.numVertices = numVertices
         self.radius = radius
         self.orientation = orientation
 
-        theta = 2*pi/self.numVertices*arange(self.numVertices) + \
+    __init__.__doc__ = cbook.dedent(__init__.__doc__) % artist.kwdocd
+
+
+
+    def get_verts(self):
+        theta = 2*npy.pi/self.numVertices*npy.arange(self.numVertices) + \
                 self.orientation
-        r = self.radius
-        xs = self.xy[0] + r*cos(theta)
-        ys = self.xy[1] + r*sin(theta)
+        r = float(self.radius)
+        x, y = map(float, self.xy)
+
+        xs = x + r*npy.cos(theta)
+        ys = y + r*npy.sin(theta)
+
+        #xs = self.convert_xunits(xs)
+        #ys = self.convert_yunits(ys)
+
 
         self.verts = zip(xs, ys)
 
-    def get_verts(self):
         return self.verts
 
 class Polygon(Patch):
     """
-    A general polygon patch.  xy is a sequence of x,y 2 tuples tuples
+    A general polygon patch.
     """
+    def __str__(self):
+        return "Poly(%g,%g)"%self.xy[0]
+
     def __init__(self, xy, **kwargs):
+        """
+        xy is a sequence of (x,y) 2 tuples
+
+        Valid kwargs are:
+        %(Patch)s
+        See Patch documentation for additional kwargs
+        """
+
         Patch.__init__(self, **kwargs)
         if not isinstance(xy, list):
             xy = list(xy)
         self.xy = xy
+    __init__.__doc__ = cbook.dedent(__init__.__doc__) % artist.kwdocd
+
+
 
     def get_verts(self):
-        return self.xy
+        xs, ys = zip(*self.xy)[:2]
+        xs = self.convert_xunits(xs)
+        ys = self.convert_yunits(ys)
+        return zip(xs, ys)
 
 
 class Wedge(Polygon):
+    def __str__(self):
+        return "Wedge(%g,%g)"%self.xy[0]
     def __init__(self, center, r, theta1, theta2,
-                 dtheta=0.1, **kwargs):
+                 dtheta=5.0, **kwargs):
         """
         Draw a wedge centered at x,y tuple center with radius r that
         sweeps theta1 to theta2 (angles)
 
-
-        kwargs are Polygon keyword args
-
         dtheta is the resolution in degrees
 
+        Valid kwargs are:
+        %(Patch)s
+
         """
         xc, yc = center
-        rads = (math.pi/180.)*arange(theta1, theta2+0.1*dtheta, dtheta)
-        xs = r*cos(rads)+xc
-        ys = r*sin(rads)+yc
+        theta1 = float(theta1)
+        theta2 = float(theta2)
+        dtheta = float(dtheta)
+        num_points = abs(theta2 - theta1) / dtheta
+        rads = (npy.pi/180.) * npy.linspace(theta1, theta2, num_points, endpoint=True)
+        xs = r*npy.cos(rads)+xc
+        ys = r*npy.sin(rads)+yc
         verts = [center]
         verts.extend([(x,y) for x,y in zip(xs,ys)])
 
         Polygon.__init__(self, verts, **kwargs)
-
+    __init__.__doc__ = cbook.dedent(__init__.__doc__) % artist.kwdocd
 
 class Arrow(Polygon):
     """
     An arrow patch
     """
+    def __str__(self):
+        x1,y1 = self.xy[0]
+        x2,y2 = self.xy[1]
+        cx,cy = (x1+x2)/2.,(y1+y2)/2.
+        return "Arrow(%g,%g)"%(cx,cy)
+
     def __init__( self, x, y, dx, dy, width=1.0, **kwargs ):
         """Draws an arrow, starting at (x,y), direction and length
         given by (dx,dy) the width of the arrow is scaled by width
-        """
-        arrow = array( [
+
+        Valid kwargs are:
+        %(Patch)s
+          """
+        arrow = npy.array( [
             [ 0.0,  0.1 ], [ 0.0, -0.1],
             [ 0.8, -0.1 ], [ 0.8, -0.3],
             [ 1.0,  0.0 ], [ 0.8,  0.3],
             [ 0.8,  0.1 ] ] )
-        L = sqrt(dx**2+dy**2) or 1 # account for div by zero
+        L = npy.sqrt(dx**2+dy**2) or 1 # account for div by zero
         arrow[:,0] *= L
         arrow[:,1] *= width
         cx = float(dx)/L
         sx = float(dy)/L
-        M = array( [ [ cx, sx],[ -sx, cx ] ] )
-        verts = matrixmultiply( arrow, M )+ [x,y]
+        M = npy.array( [ [ cx, sx],[ -sx, cx ] ] )
+        verts = npy.dot( arrow, M )+ [x,y]
         Polygon.__init__( self, [ tuple(t) for t in verts ], **kwargs )
-
+    __init__.__doc__ = cbook.dedent(__init__.__doc__) % artist.kwdocd
 
 class FancyArrow(Polygon):
     """Like Arrow, but lets you set head width and head height independently."""
 
+    def __str__(self):
+        x1,y1 = self.xy[0]
+        x2,y2 = self.xy[1]
+        cx,cy = (x1+x2)/2.,(y1+y2)/2.
+        return "FancyArrow(%g,%g)"%(cx,cy)
+
     def __init__(self, x, y, dx, dy, width=0.001, length_includes_head=False, \
         head_width=None, head_length=None, shape='full', overhang=0, \
         head_starts_at_zero=False,**kwargs):
@@ -456,13 +591,17 @@ class FancyArrow(Polygon):
 
         head_starts_at_zero: if True, the head starts being drawn at coordinate
         0 instead of ending at coordinate 0.
+
+        Valid kwargs are:
+        %(Patch)s
+
         """
         if head_width is None:
             head_width = 3 * width
         if head_length is None:
             head_length = 1.5 * head_width
 
-        distance = sqrt(dx**2 + dy**2)
+        distance = npy.sqrt(dx**2 + dy**2)
         if length_includes_head:
             length=distance
         else:
@@ -472,7 +611,7 @@ class FancyArrow(Polygon):
         else:
             #start by drawing horizontal arrow, point at (0,0)
             hw, hl, hs, lw = head_width, head_length, overhang, width
-            left_half_arrow = array([
+            left_half_arrow = npy.array([
                 [0.0,0.0],                  #tip
                 [-hl, -hw/2.0],             #leftmost
                 [-hl*(1-hs), -lw/2.0], #meets stem
@@ -493,24 +632,115 @@ class FancyArrow(Polygon):
                 if shape == 'right':
                     coords = right_half_arrow
                 elif shape == 'full':
-                    coords=concatenate([left_half_arrow,right_half_arrow[::-1]])
+                    coords=npy.concatenate([left_half_arrow,right_half_arrow[::-1]])
                 else:
                     raise ValueError, "Got unknown shape: %s" % shape
             cx = float(dx)/distance
             sx = float(dy)/distance
-            M = array([[cx, sx],[-sx,cx]])
-            verts = matrixmultiply(coords, M) + (x+dx, y+dy)
+            M = npy.array([[cx, sx],[-sx,cx]])
+            verts = npy.dot(coords, M) + (x+dx, y+dy)
 
         Polygon.__init__(self, map(tuple, verts), **kwargs)
+    __init__.__doc__ = cbook.dedent(__init__.__doc__) % artist.kwdocd
+
+class YAArrow(Polygon):
+    """
+    Yet another arrow class
+
+    This is an arrow that is defined in display space and has a tip at
+    x1,y1 and a base at x2, y2.
+    """
+    def __str__(self):
+        x1,y1 = self.xy[0]
+        x2,y2 = self.xy[1]
+        cx,cy = (x1+x2)/2.,(y1+y2)/2.
+        return "YAArrow(%g,%g)"%(cx,cy)
+
+    def __init__(self, dpi, xytip, xybase, width=4, frac=0.1, headwidth=12, **kwargs):
+        """
+        xytip : (x,y) location of arrow tip
+        xybase : (x,y) location the arrow base mid point
+        dpi : the figure dpi instance (fig.dpi)
+        width : the width of the arrow in points
+        frac  : the fraction of the arrow length occupied by the head
+        headwidth : the width of the base of the arrow head in points
+
+        Valid kwargs are:
+        %(Patch)s
+
+        """
+        self.dpi = dpi
+        self.xytip = xytip
+        self.xybase = xybase
+        self.width = width
+        self.frac = frac
+        self.headwidth = headwidth
+        verts = self.get_verts()
+        Polygon.__init__(self, verts, **kwargs)
+    __init__.__doc__ = cbook.dedent(__init__.__doc__) % artist.kwdocd
+
+
+
+    def get_verts(self):
+        # the base vertices
+        x1, y1 = self.xytip
+        x2, y2 = self.xybase
+        k1 = self.width*self.dpi.get()/72./2.
+        k2 = self.headwidth*self.dpi.get()/72./2.
+        xb1, yb1, xb2, yb2 = self.getpoints(x1, y1, x2, y2, k1)
+
+        # a point on the segment 20% of the distance from the tip to the base
+        theta = math.atan2(y2-y1, x2-x1)
+        r = math.sqrt((y2-y1)**2. + (x2-x1)**2.)
+        xm = x1 + self.frac * r * math.cos(theta)
+        ym = y1 + self.frac * r * math.sin(theta)
+        xc1, yc1, xc2, yc2 = self.getpoints(x1, y1, xm, ym, k1)
+        xd1, yd1, xd2, yd2 = self.getpoints(x1, y1, xm, ym, k2)
+
+
+        xs = self.convert_xunits([xb1, xb2, xc2, xd2, x1, xd1, xc1])
+        ys = self.convert_yunits([yb1, yb2, yc2, yd2, y1, yd1, yc1])
+        return zip(xs, ys)
+
+
+    def getpoints(self, x1,y1,x2,y2, k):
+        """
+        for line segment defined by x1,y1 and x2,y2, return the points on
+        the line that is perpendicular to the line and intersects x2,y2
+        and the distance from x2,y2 ot the returned points is k
+        """
+        x1,y1,x2,y2,k = map(float, (x1,y1,x2,y2,k))
+        m = (y2-y1)/(x2-x1)
+        pm = -1./m
+        a = 1
+        b = -2*y2
+        c = y2**2. - k**2.*pm**2./(1. + pm**2.)
+
+        y3a = (-b + math.sqrt(b**2.-4*a*c))/(2.*a)
+        x3a = (y3a - y2)/pm + x2
+
+        y3b = (-b - math.sqrt(b**2.-4*a*c))/(2.*a)
+        x3b = (y3b - y2)/pm + x2
+        return x3a, y3a, x3b, y3b
 
 
 class CirclePolygon(RegularPolygon):
     """
     A circle patch
     """
+    def __str__(self):
+        return "CirclePolygon(%d,%d)"%self.center
+
     def __init__(self, xy, radius=5,
                  resolution=20,  # the number of vertices
                  **kwargs):
+        """
+        Create a circle at xy=(x,y) with radius given by 'radius'
+
+        Valid kwargs are:
+        %(Patch)s
+
+        """
         self.center = xy
         self.radius = radius
         RegularPolygon.__init__(self, xy,
@@ -518,37 +748,106 @@ class CirclePolygon(RegularPolygon):
                                 radius,
                                 orientation=0,
                                 **kwargs)
+    __init__.__doc__ = cbook.dedent(__init__.__doc__) % artist.kwdocd
 
 
+def inellipse(x,y,cx,cy,a,b,angle):
+    x,y = x-cx,y-cy
+    theta = math.atan2(x,y) + math.radians(angle)
+    rsq = x*x+y*y
+    asin = a * math.sin(theta)
+    bcos = b * math.cos(theta)
+    Rsq = b*b*a*a / (bcos*bcos + asin*asin)
+    return Rsq > rsq;
+
 class Ellipse(Patch):
     """
     A scale-free ellipse
-    
-    xy - center of ellipse
-    width - length of horizontal axis
-    height - length of vertical axis
-    angle - rotation in degrees (anti-clockwise)
     """
+    offset = 4.0 * (npy.sqrt(2) - 1) / 3.0
+
+    circle = npy.array([
+        [-1.0, 0.0],
+
+        [-1.0, offset],
+        [-offset, 1.0],
+        [0.0, 1.0],
+
+        [offset, 1.0],
+        [1.0, offset],
+        [1.0, 0.0],
+
+        [1.0, -offset],
+        [offset, -1.0],
+        [0.0, -1.0],
+
+        [-offset, -1.0],
+        [-1.0, -offset],
+        [-1.0, 0.0],
+
+        [-1.0, 0.0]
+        ],
+                       npy.float_)
+
+    
+    def __str__(self):
+        return "Ellipse(%d,%d;%dx%d)"%(self.center[0],self.center[1],self.width,self.height)
+
     def __init__(self, xy, width, height, angle=0.0, **kwargs):
+        """
+        xy - center of ellipse
+        width - length of horizontal axis
+        height - length of vertical axis
+        angle - rotation in degrees (anti-clockwise)
+
+        Valid kwargs are:
+        %(Patch)s
+        """
         Patch.__init__(self, **kwargs)
 
-        self.center  = array(xy, Float)
+        # self.center  = npy.array(xy, npy.float)
+        self.center = xy
         self.width, self.height = width, height
         self.angle = angle
-        
-        x,y = self.center
-        l,r = x-width/2.0, x+width/2.0
-        b,t = y-height/2.0, y+height/2.0
-        
-        self.verts = array(((x,y),(l,y),(x,t),(r,y),(x,b)), Float)
-        
+
+    def contains(self,ev):
+        if ev.xdata is None or ev.ydata is None: return False,{}
+        inside = inellipse(ev.xdata,ev.ydata,
+                           self.center[0],self.center[1],
+                           self.height*0.5,self.width*0.5,self.angle)
+        return inside,{}
+
     def get_verts(self):
-        """
-        Not actually used for rendering.  Provided to conform to
-        Patch super class.
-        """
-        return self.verts
-        
+
+        xcenter, ycenter = self.center
+        width, height = self.width, self.height
+
+        xcenter = self.convert_xunits(xcenter)
+        width = self.convert_xunits(width)        
+        ycenter = self.convert_yunits(ycenter)
+        height = self.convert_xunits(height)        
+
+
+
+        angle = self.angle
+
+        theta = npy.arange(0.0, 360.0, 1.0)*npy.pi/180.0
+        x = width/2. * npy.cos(theta)
+        y = height/2. * npy.sin(theta)
+
+        rtheta = angle*npy.pi/180.
+        R = npy.array([
+            [npy.cos(rtheta),  -npy.sin(rtheta)],
+            [npy.sin(rtheta), npy.cos(rtheta)],
+            ])
+
+
+        x, y = npy.dot(R, npy.array([x, y]))
+        x += xcenter
+        y += ycenter
+
+        return zip(x, y)
+
     def draw(self, renderer):
         if not self.get_visible(): return
         #renderer.open_group('patch')
@@ -557,35 +856,113 @@ class Ellipse(Patch):
         gc.set_linewidth(self._linewidth)
         gc.set_alpha(self._alpha)
         gc.set_antialiased(self._antialiased)
-        if self.get_clip_on(): gc.set_clip_rectangle(
-            self.clipbox.get_bounds())
+        self._set_gc_clip(gc)
+
         gc.set_capstyle('projecting')
 
         if not self.fill or self._facecolor is None: rgbFace = None
-        else: rgbFace = colorConverter.to_rgb(self._facecolor)
+        else: rgbFace = colors.colorConverter.to_rgb(self._facecolor)
 
         if self._hatch:
             gc.set_hatch(self._hatch )
 
-        tverts = self._transform.seq_xy_tups(self.verts) # center is first vert
-        width = tverts[3,0] - tverts[1,0]
-        height = tverts[2,1] - tverts[4,1]
 
-        renderer.draw_arc(gc, rgbFace, tverts[0,0], tverts[0,1], width, height, 0.0, 360.0, self.angle)
+        if not hasattr(renderer, 'draw_path'):
+            mpl.verbose.report('patches.Ellipse renderer does not support path drawing; falling back on vertex approximation for nonlinear transformation')
+            renderer.draw_polygon(gc, rgbFace, self.get_verts())
+            return
         
+
+        x, y = self.center
+        x = self.convert_xunits(x)
+        y = self.convert_yunits(y)
+        w = self.convert_xunits(self.width)/2.
+        h = self.convert_yunits(self.height)/2.
+
+        theta = self.angle * npy.pi/180.
+        T = npy.array([
+            [1, 0, x],
+            [0, 1, y],
+            [0, 0, 1]])
+
+
+
+        
+        S = npy.array([
+            [w, 0, 0],
+            [0, h, 0],
+            [0, 0, 1]])
+
+
+        
+        # rotate by theta
+        R = npy.array([
+            [npy.cos(theta),  -npy.sin(theta), 0],
+            [npy.sin(theta), npy.cos(theta), 0],
+            [0,           0,          1]])
+
+        # transform unit circle into ellipse
+        E = npy.dot(T, npy.dot(R, S))
+        
+
+        # Apply the display affine
+        sx, b, c, sy, tx, ty = self.get_transform().as_vec6_val()
+
+        # display coords
+        D = npy.array([
+            [sx, b, tx],
+            [c, sy, ty],
+            [0, 0, 1]], npy.float_)
+
+        M = npy.dot(D,E)
+
+        C = npy.ones((3, len(self.circle)))
+        C[0:2,:] = self.circle.T
+        
+        ellipse = npy.dot(M, C).T[:,:2]
+
+        path =  agg.path_storage()
+        path.move_to(*ellipse[0])
+        verts = ellipse[1:4].flat
+        path.curve4(*verts)
+        verts = ellipse[4:7].flat
+        path.curve4(*verts)
+        verts = ellipse[7:10].flat
+        path.curve4(*verts)
+        verts = ellipse[10:13].flat
+        path.curve4(*verts)
+        path.close_polygon()
+
+        renderer.draw_path(gc, rgbFace, path)
+
+                              
+
 class Circle(Ellipse):
     """
     A circle patch
     """
+    def __str__(self):
+        return "Circle((%g,%g),r=%g)"%(self.center[0],self.center[1],self.radius)
+
     def __init__(self, xy, radius=5,
                  **kwargs):
+        """
+        Create true circle at center xy=(x,y) with given radius;
+        unlike circle polygon which is a polygonal approcimation, this
+        uses splines and is much closer to a scale free circle
+
+        Valid kwargs are:
+        %(Patch)s
+
+        """
         if kwargs.has_key('resolution'):
             import warnings
             warnings.warn('Circle is now scale free.  Use CirclePolygon instead!', DeprecationWarning)
-            popd(kwargs, 'resolution')
+            kwargs.pop('resolution')
 
         self.radius = radius
         Ellipse.__init__(self, xy, radius*2, radius*2, **kwargs)
+    __init__.__doc__ = cbook.dedent(__init__.__doc__) % artist.kwdocd
 
 
 class PolygonInteractor:
@@ -607,14 +984,17 @@ class PolygonInteractor:
     showverts = True
     epsilon = 5  # max pixel distance to count as a vertex hit
 
+    def __str__(self):
+        return "PolygonInteractor"
+
     def __init__(self, poly):
         if poly.figure is None:
             raise RuntimeError('You must first add the polygon to a figure or canvas before defining the interactor')
         canvas = poly.figure.canvas
         self.poly = poly
-        self.poly.verts = list(self.poly.verts)
+        self.poly.verts = list(self.poly.get_verts())
         x, y = zip(*self.poly.verts)
-        self.line = Line2D(x,y,marker='o', markerfacecolor='r')
+        self.line = lines.Line2D(x,y,marker='o', markerfacecolor='r')
         #self._update_line(poly)
 
         cid = self.poly.add_callback(self.poly_changed)
@@ -631,7 +1011,7 @@ class PolygonInteractor:
         'this method is called whenever the polygon object is called'
         # only copy the artist props to the line (except visibility)
         vis = self.line.get_visible()
-        Artist.update_from(self.line, poly)
+        artist.Artist.update_from(self.line, poly)
         self.line.set_visible(vis)  # don't use the poly visibility state
 
 
@@ -641,9 +1021,8 @@ class PolygonInteractor:
 
         # display coords
         xt, yt = self.poly.get_transform().numerix_x_y(x, y)
-        d = sqrt((xt-event.x)**2 + (yt-event.y)**2)
-        indseq = nonzero(equal(d, amin(d)))
-        ind = indseq[0]
+        d = npy.sqrt((xt-event.x)**2 + (yt-event.y)**2)
+        ind, = npy.nonzero(npy.equal(d, npy.amin(d)))
 
         if d[ind]>=self.epsilon:
             ind = None
@@ -681,7 +1060,7 @@ class PolygonInteractor:
             for i in range(len(xys)-1):
                 s0 = xys[i]
                 s1 = xys[i+1]
-                d = dist_point_to_segment(p, s0, s1)
+                d = mlab.dist_point_to_segment(p, s0, s1)
                 if d<=self.epsilon:
                     self.poly.verts.insert(i+1, (event.xdata, event.ydata))
                     self.line.set_data(zip(*self.poly.verts))
@@ -713,7 +1092,7 @@ def bbox_artist(artist, renderer, props=None, fill=True):
     """
     if props is None: props = {}
     props = props.copy() # don't want to alter the pad externally
-    pad = popd(props, 'pad', 4)
+    pad = props.pop('pad', 4)
     pad = renderer.points_to_pixels(pad)
     bbox = artist.get_window_extent(renderer)
     l,b,w,h = bbox.get_bounds()
@@ -749,3 +1128,10 @@ def draw_bbox(bbox, renderer, color='k', trans=None):
     r.set_clip_on( False )
     r.draw(renderer)
 
+artist.kwdocd['Patch'] = patchdoc = artist.kwdoc(Patch)
+
+for k in ('Rectangle', 'Circle', 'RegularPolygon', 'Polygon', 'Wedge', 'Arrow',
+          'FancyArrow', 'YAArrow', 'CirclePolygon', 'Ellipse'):
+    artist.kwdocd[k] = patchdoc
+
+
