@@ -21,9 +21,10 @@ License   : matplotlib license
 """
 from __future__ import division
 import sys, warnings
-from matplotlib import verbose
-from numerix import ones, Float, add, asarray
 
+from matplotlib import verbose
+
+import artist
 from artist import Artist
 from patches import Rectangle
 from cbook import enumerate, is_string_like, flatten
@@ -46,6 +47,7 @@ class Cell(Rectangle):
                  fill=True,
                  text='',
                  loc=None,
+                 fontproperties=None
                  ):
 
         # Call base
@@ -57,7 +59,8 @@ class Cell(Rectangle):
         # Create text object
         if loc is None: loc = 'right'
         self._loc = loc
-        self._text = Text(x=xy[0], y=xy[1], text=text)
+        self._text = Text(x=xy[0], y=xy[1], text=text,
+                          fontproperties=fontproperties)
         self._text.set_clip_on(False)
 
 
@@ -129,7 +132,7 @@ class Cell(Rectangle):
     def get_text_bounds(self, renderer):
         """ Get text bounds in axes co-ordinates. """
         bbox = self._text.get_window_extent(renderer)
-        bboxa = inverse_transform_bbox(self._transform, bbox)
+        bboxa = inverse_transform_bbox(self.get_transform(), bbox)
         return bboxa.get_bounds()
 
     def get_required_width(self, renderer):
@@ -201,13 +204,15 @@ class Table(Artist):
         self._autoColumns = []
         self._autoFontsize = True
 
+        self._cachedRenderer = None
+
     def add_cell(self, row, col, *args, **kwargs):
         """ Add a cell to the table. """
         xy = (0,0)
 
         cell = Cell(xy, *args, **kwargs)
         cell.set_figure(self.figure)
-        cell.set_transform(self._transform)
+        cell.set_transform(self.get_transform())
 
         cell.set_clip_on(False)
         self._cells[(row, col)] = cell
@@ -216,6 +221,13 @@ class Table(Artist):
         return self.FONTSIZE/72.0*self.figure.dpi.get()/self._axes.bbox.height() * 1.2
 
     def draw(self, renderer):
+        # Need a renderer to do hit tests on mouseevent; assume the last one will do
+        if renderer is None:
+            renderer = self._cachedRenderer
+        if renderer is None:
+            raise RuntimeError('No renderer defined')
+        self._cachedRenderer = renderer
+
         if not self.get_visible(): return
         renderer.open_group('table')
         self._update_positions(renderer)
@@ -237,11 +249,30 @@ class Table(Artist):
                  if pos[0] >= 0 and pos[1] >= 0]
 
         bbox = bbox_all(boxes)
-        return inverse_transform_bbox(self._transform, bbox)
+        return inverse_transform_bbox(self.get_transform(), bbox)
 
-    def get_child_artists(self):
-        'Return the Artists cintained by the table'
+    def contains(self,mouseevent):
+        """Test whether the mouse event occurred in the table.
+
+        Returns T/F, {}
+        """
+        if callable(self._contains): return self._contains(self,mouseevent)
+
+        # TODO: Return index of the cell containing the cursor so that the user
+        # doesn't have to bind to each one individually.
+        if self._cachedRenderer is not None:
+            boxes = [self._cells[pos].get_window_extent(self._cachedRenderer)
+                 for pos in self._cells.keys()
+                 if pos[0] >= 0 and pos[1] >= 0]
+            bbox = bbox_all(boxes)
+            return bbox.contains(mouseevent.x,mouseevent.y),{}
+        else:
+            return False,{}
+
+    def get_children(self):
+        'Return the Artists contained by the table'
         return self._cells.values()
+    get_child_artists = get_children  # backward compatibility
 
     def get_window_extent(self, renderer):
         'Return the bounding box of the table in window coords'
@@ -333,9 +364,10 @@ class Table(Artist):
 
     def set_fontsize(self, size):
         """
-Set the fontsize of the cell text
+        Set the fontsize of the cell text
 
-ACCEPTS: a float in points"""
+        ACCEPTS: a float in points
+        """
 
         for cell in self._cells.itervalues():
             cell.set_fontsize(size)
@@ -510,3 +542,4 @@ def table(ax,
     return table
 
 
+artist.kwdocd['Table'] = artist.kwdoc(Table)
