@@ -11,20 +11,14 @@
     was written by following the Python "Extending and Embedding"
     tutorial.
 
-  $Id: cntr.c,v 1.3 2005/06/02 22:02:32 jdh2358 Exp $
+  $Id: cntr.c 3439 2007-06-30 22:20:16Z nnemec $
  */
 
 #include <Python.h>
 #include "structmember.h"
 #include <stdlib.h>
 #include <stdio.h>
-
-#ifdef NUMARRAY
-#include "numarray/arrayobject.h"
-#else
-#include "Numeric/arrayobject.h"
-#endif
-
+#include "numerix.h"
 
 /* Note that all arrays in these routines are Fortran-style,
    in the sense that the "i" index varies fastest; the dimensions
@@ -1328,6 +1322,11 @@ build_cntr_list_p(long *np, double *xp, double *yp, int nparts, long ntotal)
     return NULL;
 }
 
+
+#if 0
+/* the following function is not used, so it produces a warning
+ * commented it out NN - 070630 */
+
 /* Build a list of tuples (X, Y), where X and Y are 1-D arrays. */
 static PyObject *
 build_cntr_list_v(long *np, double *xp, double *yp, int nparts, long ntotal)
@@ -1344,8 +1343,8 @@ build_cntr_list_v(long *np, double *xp, double *yp, int nparts, long ntotal)
     for (i = 0; i < nparts; i++)
     {
         dims[0] = np[i];
-        xv = (PyArrayObject *) PyArray_FromDims(1, dims, 'd');
-        yv = (PyArrayObject *) PyArray_FromDims(1, dims, 'd');
+        xv = (PyArrayObject *) PyArray_FromDims(1, dims, PyArray_DOUBLE);
+        yv = (PyArrayObject *) PyArray_FromDims(1, dims, PyArray_DOUBLE);
         if (xv == NULL || yv == NULL)  goto error;
         for (j = 0; j < dims[0]; j++)
         {
@@ -1363,6 +1362,42 @@ build_cntr_list_v(long *np, double *xp, double *yp, int nparts, long ntotal)
     Py_XDECREF(all_contours);
     return NULL;
 }
+#endif
+
+/* Build a list of XY 2-D arrays, shape (N,2) */
+static PyObject *
+build_cntr_list_v2(long *np, double *xp, double *yp, int nparts, long ntotal)
+{
+    PyObject *all_contours;
+    PyArrayObject *xyv;
+    int dims[2];
+    int i;
+    long j, k;
+
+    all_contours = PyList_New(nparts);
+
+    k = 0;
+    for (i = 0; i < nparts; i++)
+    {
+        dims[0] = np[i];
+        dims[1] = 2;
+        xyv = (PyArrayObject *) PyArray_FromDims(2, dims, PyArray_DOUBLE);
+        if (xyv == NULL)  goto error;
+        for (j = 0; j < dims[0]; j++)
+        {
+            ((double *)xyv->data)[2*j] = xp[k];
+            ((double *)xyv->data)[2*j+1] = yp[k];
+            k++;
+        }
+        if (PyList_SetItem(all_contours, i, (PyObject *)xyv)) goto error;
+    }
+    return all_contours;
+
+    error:
+    Py_XDECREF(all_contours);
+    return NULL;
+}
+
 
 
 /* cntr_trace is called once per contour level or level pair.
@@ -1373,7 +1408,7 @@ build_cntr_list_v(long *np, double *xp, double *yp, int nparts, long ntotal)
 */
 
 PyObject *
-cntr_trace(Csite *site, double levels[], int nlevels, int points)
+cntr_trace(Csite *site, double levels[], int nlevels, int points, long nchunk)
 {
     PyObject *c_list;
     double *xp0;
@@ -1381,7 +1416,7 @@ cntr_trace(Csite *site, double levels[], int nlevels, int points)
     long *nseg0;
     int iseg;
 
-    long nchunk = 30; /* hardwired for now */
+    /* long nchunk = 30; was hardwired */
     long n;
     long nparts = 0;
     long ntotal = 0;
@@ -1458,7 +1493,7 @@ cntr_trace(Csite *site, double levels[], int nlevels, int points)
     }
     else
     {
-        c_list = build_cntr_list_v(nseg0, xp0, yp0, nparts, ntotal);
+        c_list = build_cntr_list_v2(nseg0, xp0, yp0, nparts, ntotal);
     }
     PyMem_Free(xp0); PyMem_Free(yp0); PyMem_Free(nseg0);
     site->xcp = NULL; site->ycp = NULL;
@@ -1544,13 +1579,13 @@ Cntr_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 static int
 Cntr_init(Cntr *self, PyObject *args, PyObject *kwds)
 {
+    static char *kwlist[] = {"x", "y", "z", "mask", NULL};
     PyObject *xarg, *yarg, *zarg, *marg;
     PyArrayObject *xpa, *ypa, *zpa, *mpa;
     long iMax, jMax;
     char *mask;
 
     marg = NULL;
-    static char *kwlist[] = {"x", "y", "z", "mask", NULL};
 
     if (! PyArg_ParseTupleAndKeywords(args, kwds, "OOO|O", kwlist,
                                       &xarg, &yarg, &zarg, &marg))
@@ -1566,18 +1601,25 @@ Cntr_init(Cntr *self, PyObject *args, PyObject *kwds)
         return -1;
     }
 
-    xpa = (PyArrayObject *) PyArray_ContiguousFromObject(xarg, 'd', 2, 2);
-    ypa = (PyArrayObject *) PyArray_ContiguousFromObject(yarg, 'd', 2, 2);
-    zpa = (PyArrayObject *) PyArray_ContiguousFromObject(zarg, 'd', 2, 2);
+    xpa = (PyArrayObject *) PyArray_ContiguousFromObject(xarg,
+							 PyArray_DOUBLE, 2, 2);
+    ypa = (PyArrayObject *) PyArray_ContiguousFromObject(yarg,
+							 PyArray_DOUBLE,
+							 2, 2);
+    zpa = (PyArrayObject *) PyArray_ContiguousFromObject(zarg, PyArray_DOUBLE,
+							 2, 2);
     if (marg)
-        mpa = (PyArrayObject *) PyArray_ContiguousFromObject(marg, '1', 2, 2);
+        mpa = (PyArrayObject *) PyArray_ContiguousFromObject(marg,
+							     PyArray_SBYTE,
+							     2, 2);
     else
         mpa = NULL;
 
     if (xpa == NULL || ypa == NULL || zpa == NULL || (marg && mpa == NULL))
     {
         PyErr_SetString(PyExc_ValueError,
-            "Arguments x, y, z, mask (if present) must be 2D arrays.");
+            "Arguments x, y, z, mask (if present) must be 2D arrays.\n"
+            "x, y, z must be castable to double.");
         goto error;
     }
     iMax = zpa->dimensions[1];
@@ -1621,16 +1663,17 @@ Cntr_trace(Cntr *self, PyObject *args, PyObject *kwds)
     double levels[2] = {0.0, -1e100};
     int nlevels = 2;
     int points = 0;
-    static char *kwlist[] = {"level0", "level1", "points", NULL};
+    long nchunk = 0L;
+    static char *kwlist[] = {"level0", "level1", "points", "nchunk", NULL};
 
-    if (! PyArg_ParseTupleAndKeywords(args, kwds, "d|di", kwlist,
-                                      levels, levels+1, &points))
+    if (! PyArg_ParseTupleAndKeywords(args, kwds, "d|dil", kwlist,
+                                      levels, levels+1, &points, &nchunk))
     {
         return NULL;
     }
     if (levels[1] == -1e100 || levels[1] <= levels[0])
         nlevels = 1;
-    return cntr_trace(self->site, levels, nlevels, points);
+    return cntr_trace(self->site, levels, nlevels, points, nchunk);
 }
 
 static PyMethodDef Cntr_methods[] = {
@@ -1642,6 +1685,8 @@ static PyMethodDef Cntr_methods[] = {
      "        the levels.\n"
      "    Optional argument: points; if 0 (default), return a list of\n"
      "        vector pairs; otherwise, return a list of lists of points.\n"
+     "    Optional argument: nchunk; approximate number of grid points\n"
+     "        per chunk. 0 (default) for no chunking.\n"
     },
     {NULL}  /* Sentinel */
 };
@@ -1692,18 +1737,16 @@ static PyMethodDef module_methods[] = {
     {NULL}  /* Sentinel */
 };
 
-
-#ifdef NUMARRAY
 PyMODINIT_FUNC
-init_na_cntr(void)
+init_cntr(void)
 {
     PyObject* m;
 
     if (PyType_Ready(&CntrType) < 0)
         return;
 
-    m = Py_InitModule3("_na_cntr", module_methods,
-                       "Contouring engine as an extension type (numarray).");
+    m = Py_InitModule3("_cntr", module_methods,
+                       "Contouring engine as an extension type (numpy).");
 
     if (m == NULL)
       return;
@@ -1713,26 +1756,4 @@ init_na_cntr(void)
     PyModule_AddObject(m, "Cntr", (PyObject *)&CntrType);
 }
 
-#else
-PyMODINIT_FUNC
-init_nc_cntr(void)
-{
-    PyObject* m;
-
-    if (PyType_Ready(&CntrType) < 0)
-        return;
-
-    m = Py_InitModule3("_nc_cntr", module_methods,
-                       "Contouring engine as an extension type (Numeric).");
-
-    if (m == NULL)
-      return;
-
-    import_array();
-    Py_INCREF(&CntrType);
-    PyModule_AddObject(m, "Cntr", (PyObject *)&CntrType);
-}
-
-#endif
-
 
