@@ -5,10 +5,341 @@
 #define FIXED_MAJOR(val) (*((short *) &val+1))
 #define FIXED_MINOR(val) (*((short *) &val+0))
 
+/**
+ To improve the hinting of the fonts, this code uses a hack
+ presented here:
+ 
+ http://antigrain.com/research/font_rasterization/index.html
+ 
+ The idea is to limit the effect of hinting in the x-direction, while
+ preserving hinting in the y-direction.  Since freetype does not
+ support this directly, the dpi in the x-direction is set higher than
+ in the y-direction, which affects the hinting grid.  Then, a global
+ transform is placed on the font to shrink it back to the desired
+ size.  While it is a bit surprising that the dpi setting affects
+ hinting, whereas the global transform does not, this is documented
+ behavior of freetype, and therefore hopefully unlikely to change.
+ The freetype 2 tutorial says:
+ 
+      NOTE: The transformation is applied to every glyph that is
+      loaded through FT_Load_Glyph and is completely independent of
+      any hinting process. This means that you won't get the same
+      results if you load a glyph at the size of 24 pixels, or a glyph
+      at the size at 12 pixels scaled by 2 through a transform,
+      because the hints will have been computed differently (except
+      you have disabled hints).
+
+ This hack is enabled only when VERTICAL_HINTING is defined, and will
+ only be effective when load_char and set_text are called with 'flags=
+ LOAD_DEFAULT', which is the default.
+ */
+#define VERTICAL_HINTING
+#ifdef VERTICAL_HINTING
+#define HORIZ_HINTING 8
+#else
+#define HORIZ_HINTING 1
+#endif
+
 FT_Library _ft2Library;
 
-FT2Image::FT2Image() : bRotated(false), buffer(NULL)  {}
-FT2Image::~FT2Image() {delete [] buffer; buffer=NULL;}
+// FT2Image::FT2Image() : 
+//   _isDirty(true),
+//   _buffer(NULL),
+//   _width(0), _height(0),
+//   _rgbCopy(NULL),
+//   _rgbaCopy(NULL) {
+//   _VERBOSE("FT2Image::FT2Image");
+// }
+
+FT2Image::FT2Image(unsigned long width, unsigned long height) :
+  _isDirty(true),
+  _buffer(NULL), 
+  _width(0), _height(0),
+  _rgbCopy(NULL),
+  _rgbaCopy(NULL) {
+  _VERBOSE("FT2Image::FT2Image");
+  resize(width, height);
+}
+
+FT2Image::~FT2Image() { 
+  _VERBOSE("FT2Image::~FT2Image");
+  delete [] _buffer; 
+  _buffer=NULL; 
+  delete _rgbCopy;
+  delete _rgbaCopy;
+}
+
+void FT2Image::resize(unsigned long width, unsigned long height) {
+  size_t numBytes = width*height;
+
+  if (width != _width || height != _height) {
+    if (numBytes > _width*_height) {
+      delete [] _buffer;
+      _buffer = NULL;
+      _buffer = new unsigned char [numBytes];
+    }
+
+    _width = width;
+    _height = height;
+  }
+
+  memset(_buffer, 0, numBytes);
+
+  _isDirty = true;
+}
+
+void
+FT2Image::draw_bitmap( FT_Bitmap*  bitmap,
+	 	       FT_Int      x,
+ 		       FT_Int      y) {
+  _VERBOSE("FT2Image::draw_bitmap");
+  FT_Int image_width = (FT_Int)_width;
+  FT_Int image_height = (FT_Int)_height;
+  FT_Int char_width =  bitmap->width;
+  FT_Int char_height = bitmap->rows;
+
+  FT_Int x1 = CLAMP(x, 0, image_width);
+  FT_Int y1 = CLAMP(y, 0, image_height);
+  FT_Int x2 = CLAMP(x + char_width, 0, image_width);
+  FT_Int y2 = CLAMP(y + char_height, 0, image_height);
+
+  FT_Int x_start = MAX(0, -x);
+  FT_Int y_offset = y1 - MAX(0, -y);
+
+  for ( FT_Int i = y1; i < y2; ++i ) {
+    unsigned char* dst = _buffer + (i * image_width + x1);
+    unsigned char* src = bitmap->buffer + (((i - y_offset) * bitmap->pitch) + x_start);
+    for ( FT_Int j = x1; j < x2; ++j, ++dst, ++src )
+      *dst |= *src;
+  }
+
+  _isDirty = true;
+}
+
+void FT2Image::write_bitmap(const char* filename) const {
+  FILE *fh = fopen(filename, "w");
+
+  for ( size_t i = 0; i< _height; i++) {
+    for ( size_t j = 0; j < _width; ++j) {
+      if (_buffer[j + i*_width])
+	fputc('#', fh);
+      else
+	fputc(' ', fh);
+    }
+    fputc('\n', fh);
+  }
+
+  fclose(fh);
+}
+
+char FT2Image::write_bitmap__doc__[] =
+"write_bitmap(fname)\n"
+"\n"
+"Write the bitmap to file fname\n"
+;
+Py::Object
+FT2Image::py_write_bitmap(const Py::Tuple & args) {
+  _VERBOSE("FT2Image::write_bitmap");
+
+  args.verify_length(1);
+
+  std::string filename = Py::String(args[0]);
+
+  write_bitmap(filename.c_str());
+
+  return Py::Object();
+}
+
+void
+FT2Image::draw_rect(unsigned long x0, unsigned long y0, 
+		    unsigned long x1, unsigned long y1) {
+  if ( x0<0 || y0<0 || x1<0 || y1<0 ||
+       x0>_width || x1>_width ||
+       y0>_height || y1>_height )
+    throw Py::ValueError("Rect coords outside image bounds");
+
+  size_t top = y0*_width;
+  size_t bottom = y1*_width;
+  for (size_t i=x0; i<x1+1; ++i) {
+    _buffer[i + top] = 255;
+    _buffer[i + bottom] = 255;
+  }
+
+  for (size_t j=y0+1; j<y1; ++j) {
+    _buffer[x0 + j*_width] = 255;
+    _buffer[x1 + j*_width] = 255;
+  }
+
+  _isDirty = true;
+}
+
+char FT2Image::draw_rect__doc__[] =
+"draw_rect(x0, y0, x1, y1)\n"
+"\n"
+"Draw a rect to the image.\n"
+"\n"
+;
+Py::Object
+FT2Image::py_draw_rect(const Py::Tuple & args) {
+  _VERBOSE("FT2Image::draw_rect");
+
+  args.verify_length(4);
+
+  long x0 = Py::Int(args[0]);
+  long y0 = Py::Int(args[1]);
+  long x1 = Py::Int(args[2]);
+  long y1 = Py::Int(args[3]);
+
+  draw_rect(x0, y0, x1, y1);
+
+  return Py::Object();
+}
+
+void FT2Image::draw_rect_filled(unsigned long x0, unsigned long y0, 
+				unsigned long x1, unsigned long y1) {
+  x0 = CLAMP(x0, 0, _width);
+  y0 = CLAMP(y0, 0, _height);
+  x1 = CLAMP(x1, 0, _width);
+  y1 = CLAMP(y1, 0, _height);
+
+  for (size_t j=y0; j<y1+1; j++) {
+    for (size_t i=x0; i<x1+1; i++) {
+      _buffer[i + j*_width] = 255;
+    }
+  }
+
+  _isDirty = true;
+}  
+
+char FT2Image::draw_rect_filled__doc__[] =
+"draw_rect_filled(x0, y0, x1, y1)\n"
+"\n"
+"Draw a filled rect to the image.\n"
+"\n"
+;
+Py::Object
+FT2Image::py_draw_rect_filled(const Py::Tuple & args) {
+  _VERBOSE("FT2Image::draw_rect_filled");
+
+  args.verify_length(4);
+
+  long x0 = Py::Int(args[0]);
+  long y0 = Py::Int(args[1]);
+  long x1 = Py::Int(args[2]);
+  long y1 = Py::Int(args[3]);
+
+  draw_rect_filled(x0, y0, x1, y1);
+
+  return Py::Object();
+}
+
+char FT2Image::as_str__doc__[] =
+"width, height, s = image_as_str()\n"
+"\n"
+"Return the image buffer as a string\n"
+"\n"
+;
+Py::Object
+FT2Image::py_as_str(const Py::Tuple & args) {
+  _VERBOSE("FT2Image::as_str");
+  args.verify_length(0);
+
+  return Py::asObject
+    (PyString_FromStringAndSize((const char *)_buffer, 
+				_width*_height)
+     );
+}
+
+void FT2Image::makeRgbCopy() {
+  if (!_isDirty)
+    return;
+
+  if (!_rgbCopy) {
+    _rgbCopy = new FT2Image(_width * 3, _height);
+  } else {
+    _rgbCopy->resize(_width * 3, _height);
+  }
+  unsigned char *src		= _buffer;
+  unsigned char *src_end	= src + (_width * _height);
+  unsigned char *dst		= _rgbCopy->_buffer;
+
+  unsigned char tmp;
+  while (src != src_end) {
+    tmp = 255 - *src++;
+    *dst++ = tmp;
+    *dst++ = tmp;
+    *dst++ = tmp;
+  }
+}
+
+char FT2Image::as_rgb_str__doc__[] =
+"width, height, s = image_as_rgb_str()\n"
+"\n"
+"Return the image buffer as a 24-bit RGB string.\n"
+"\n"
+;
+Py::Object
+FT2Image::py_as_rgb_str(const Py::Tuple & args) {
+  _VERBOSE("FT2Image::as_str_rgb");
+  args.verify_length(0);
+
+  makeRgbCopy();
+  
+  return _rgbCopy->py_as_str(args);
+}
+
+void FT2Image::makeRgbaCopy() {
+  if (!_isDirty)
+    return;
+
+  if (!_rgbaCopy) {
+    _rgbaCopy = new FT2Image(_width * 4, _height);
+  } else {
+    _rgbaCopy->resize(_width * 4, _height);
+  }
+  unsigned char *src		= _buffer;
+  unsigned char *src_end	= src + (_width * _height);
+  unsigned char *dst		= _rgbaCopy->_buffer;
+
+  while (src != src_end) {
+    // We know the array has already been zero'ed out in
+    // the resize method, so we just skip over the r, g and b.
+    dst += 3;
+    *dst++ = *src++;
+  }
+}
+
+char FT2Image::as_rgba_str__doc__[] =
+"width, height, s = image_as_rgb_str()\n"
+"\n"
+"Return the image buffer as a 32-bit RGBA string.\n"
+"\n"
+;
+Py::Object
+FT2Image::py_as_rgba_str(const Py::Tuple & args) {
+  _VERBOSE("FT2Image::as_str_rgba");
+  args.verify_length(0);
+
+  makeRgbaCopy();
+  
+  return _rgbaCopy->py_as_str(args);
+}
+
+Py::Object
+FT2Image::py_get_width(const Py::Tuple & args) {
+  _VERBOSE("FT2Image::get_width");
+  args.verify_length(0);
+
+  return Py::Int((long)get_width());
+}
+
+Py::Object
+FT2Image::py_get_height(const Py::Tuple & args) {
+  _VERBOSE("FT2Image::get_height");
+  args.verify_length(0);
+
+  return Py::Int((long)get_height());
+}
 
 Glyph::Glyph( const FT_Face& face, const FT_Glyph& glyph, size_t ind) :
   glyphInd(ind) {
@@ -17,12 +348,12 @@ Glyph::Glyph( const FT_Face& face, const FT_Glyph& glyph, size_t ind) :
   FT_BBox bbox;
   FT_Glyph_Get_CBox( glyph, ft_glyph_bbox_subpixels, &bbox );
 
-  setattr("width",        Py::Int( face->glyph->metrics.width) );
+  setattr("width",        Py::Int( face->glyph->metrics.width / HORIZ_HINTING) );
   setattr("height",       Py::Int( face->glyph->metrics.height) );
-  setattr("horiBearingX", Py::Int( face->glyph->metrics.horiBearingX) );
+  setattr("horiBearingX", Py::Int( face->glyph->metrics.horiBearingX / HORIZ_HINTING) );
   setattr("horiBearingY", Py::Int( face->glyph->metrics.horiBearingY) );
   setattr("horiAdvance",  Py::Int( face->glyph->metrics.horiAdvance) );
-  setattr("linearHoriAdvance",  Py::Int( face->glyph->linearHoriAdvance) );
+  setattr("linearHoriAdvance",  Py::Int( face->glyph->linearHoriAdvance / HORIZ_HINTING) );
   setattr("vertBearingX", Py::Int( face->glyph->metrics.vertBearingX) );
 
   setattr("vertBearingY", Py::Int( face->glyph->metrics.vertBearingY) );
@@ -70,7 +401,6 @@ Glyph::get_path( const FT_Face& face) {
   //get the glyph as a path, a list of (COMMAND, *args) as desribed in matplotlib.path
   // this code is from agg's decompose_ft_outline with minor modifications
 
-
   enum {MOVETO, LINETO, CURVE3, CURVE4, ENDPOLY};
   FT_Outline& outline = face->glyph->outline;
   Py::List path;
@@ -310,12 +640,12 @@ Glyph::get_path( const FT_Face& face) {
 }
 
 
-FT2Font::FT2Font(std::string facefile)
+FT2Font::FT2Font(std::string facefile) :
+  image(NULL)
 {
   _VERBOSE(Printf("FT2Font::FT2Font %s", facefile.c_str()).str());
   clear(Py::Tuple(0));
 
-
   int error = FT_New_Face( _ft2Library, facefile.c_str(), 0, &face );
 
 
@@ -341,7 +671,13 @@ FT2Font::FT2Font(std::string facefile)
   }
 
   // set a default fontsize 12 pt at 72dpi
+#ifdef VERTICAL_HINTING  
+  error = FT_Set_Char_Size( face, 12 * 64, 0, 72 * HORIZ_HINTING, 72 );
+  static FT_Matrix transform = { 65536 / HORIZ_HINTING, 0, 0, 65536 };
+  FT_Set_Transform( face, &transform, 0 );
+#else
   error = FT_Set_Char_Size( face, 12 * 64, 0, 72, 72 );
+#endif
   //error = FT_Set_Char_Size( face, 20 * 64, 0, 80, 80 );
   if (error) {
     std::ostringstream s;
@@ -398,81 +734,27 @@ FT2Font::FT2Font(std::string facefile)
     setattr("underline_thickness", Py::Int(face->underline_thickness));
   }
 
+  setattr("fname", Py::String(facefile));
+
   _VERBOSE("FT2Font::FT2Font done");
 }
 
 FT2Font::~FT2Font()
 {
   _VERBOSE("FT2Font::~FT2Font");
-  FT_Done_Face    ( face );
 
-  delete [] image.buffer ;
-  image.buffer = NULL;
+  Py_XDECREF(image);
+  FT_Done_Face    ( face );
 
   for (size_t i=0; i<glyphs.size(); i++) {
     FT_Done_Glyph( glyphs[i] );
   }
+
   for (size_t i=0; i<gms.size(); i++) {
     Py_DECREF(gms[i]);
   }
 }
 
-
-char  FT2Font::horiz_image_to_vert_image__doc__[] =
-"horiz_image_to_vert_image()\n"
-"\n"
-"Copies the horizontal image (w, h) into a\n"
-"new image of size (h,w)\n"
-"This is equivalent to rotating the original image\n"
-"by 90 degrees ccw\n"
-;
-
-Py::Object
-FT2Font::horiz_image_to_vert_image(const Py::Tuple & args) {
-
-  // If we have already rotated, just return.
-
-  if (image.bRotated)
-    return Py::Object();
-
-
-  long width = image.width, height = image.height;
-
-  long newWidth  = image.height;
-  long newHeight = image.width;
-
-  long numBytes = image.width * image.height;
-
-  unsigned char * buffer = new unsigned char [numBytes];
-
-  long  i, j, k, offset, nhMinusOne;
-
-  nhMinusOne = newHeight-1;
-
-  for (i=0; i<height; i++) {
-
-    offset = i*width;
-
-    for (j=0; j<width; j++) {
-
-      k = nhMinusOne - j;
-
-      buffer[i + k*newWidth] = image.buffer[j + offset];
-
-    }
-
-  }
-
-  delete [] image.buffer;
-  image.buffer = buffer;
-  image.width = newWidth;
-  image.height = newHeight;
-  image.bRotated = true;
-
-  return Py::Object();
-
-}
-
 int
 FT2Font::setattr( const char *name, const Py::Object &value ) {
   _VERBOSE("FT2Font::setattr");
@@ -487,34 +769,6 @@ FT2Font::getattr( const char *name ) {
   else return getattr_default( name );
 }
 
-char FT2Font::set_bitmap_size__doc__[] =
-"set_bitmap_size(w, h)\n"
-"\n"
-"Manually set the bitmap size to render the glyps to.  This is useful"
-"in cases where you want to render several different glyphs to the bitmap"
-;
-
-Py::Object
-FT2Font::set_bitmap_size(const Py::Tuple & args) {
-  _VERBOSE("FT2Font::set_bitmap_size");
-  args.verify_length(2);
-
-  long width = Py::Int(args[0]);
-  long height = Py::Int(args[1]);
-
-  image.width   = (unsigned)width;
-  image.height  = (unsigned)height;
-
-  long numBytes = image.width * image.height;
-
-  delete [] image.buffer;
-  image.buffer = new unsigned char [numBytes];
-  for (long n=0; n<numBytes; n++)
-    image.buffer[n] = 0;
-
-  return Py::Object();
-}
-
 char FT2Font::clear__doc__[] =
 "clear()\n"
 "\n"
@@ -526,13 +780,8 @@ FT2Font::clear(const Py::Tuple & args) {
   _VERBOSE("FT2Font::clear");
   args.verify_length(0);
 
-  //todo: move to image method?
-  delete [] image.buffer ;
-  image.buffer  = NULL;
-  image.width   = 0;
-  image.height  = 0;
-  image.offsetx = 0;
-  image.offsety = 0;
+  Py_XDECREF(image);
+  image = NULL;
 
   angle = 0.0;
 
@@ -570,9 +819,17 @@ FT2Font::set_size(const Py::Tuple & args) {
   double ptsize = Py::Float(args[0]);
   double dpi = Py::Float(args[1]);
 
+#ifdef VERTICAL_HINTING
+  int error = FT_Set_Char_Size( face, (long)(ptsize * 64), 0,
+				(unsigned int)dpi * HORIZ_HINTING,
+				(unsigned int)dpi );
+  static FT_Matrix transform = { 65536 / HORIZ_HINTING, 0, 0, 65536 };
+  FT_Set_Transform( face, &transform, 0 );
+#else
   int error = FT_Set_Char_Size( face, (long)(ptsize * 64), 0,
 				(unsigned int)dpi,
 				(unsigned int)dpi );
+#endif  
   if (error)
     throw Py::RuntimeError("Could not set the fontsize");
   return Py::Object();
@@ -646,7 +903,7 @@ FT2Font::get_kerning(const Py::Tuple & args) {
   FT_Vector delta;
 
   if (!FT_Get_Kerning( face, left, right, mode, &delta )) {
-    return Py::Int(delta.x);
+    return Py::Int(delta.x / HORIZ_HINTING);
   }
   else {
     return Py::Int(0);
@@ -663,7 +920,7 @@ char FT2Font::set_text__doc__[] =
 "You must call this before draw_glyphs_to_bitmap\n"
 "A sequence of x,y positions is returned";
 Py::Object
-FT2Font::set_text(const Py::Tuple & args) {
+FT2Font::set_text(const Py::Tuple & args, const Py::Dict & kwargs) {
   _VERBOSE("FT2Font::set_text");
   args.verify_length(2);
 
@@ -685,6 +942,11 @@ FT2Font::set_text(const Py::Tuple & args) {
   angle = Py::Float(args[1]);
 
   angle = angle/360.0*2*3.14159;
+
+  long flags = FT_LOAD_FORCE_AUTOHINT;
+  if (kwargs.hasKey("flags"))
+    flags = Py::Long(kwargs["flags"]);
+
   //this computes width and height in subpixels so we have to divide by 64
   matrix.xx = (FT_Fixed)( cos( angle ) * 0x10000L );
   matrix.xy = (FT_Fixed)(-sin( angle ) * 0x10000L );
@@ -719,9 +981,9 @@ FT2Font::set_text(const Py::Tuple & args) {
       FT_Vector delta;
       FT_Get_Kerning( face, previous, glyph_index,
 		      FT_KERNING_DEFAULT, &delta );
-      pen.x += delta.x;
+      pen.x += delta.x / HORIZ_HINTING;
     }
-    error = FT_Load_Glyph( face, glyph_index, FT_LOAD_DEFAULT );
+    error = FT_Load_Glyph( face, glyph_index, flags );
     if ( error ) {
       std::cerr << "\tcould not load glyph for " << thischar << std::endl;
       continue;
@@ -774,7 +1036,7 @@ FT2Font::get_glyph(const Py::Tuple & args){
   if ( (size_t)num >= gms.size())
     throw Py::ValueError("Glyph index out of range");
 
-  //todo: refcount?
+  Py_INCREF(gms[num]);
   return Py::asObject(gms[num]);
 }
 
@@ -792,7 +1054,7 @@ FT2Font::get_num_glyphs(const Py::Tuple & args){
 }
 
 char FT2Font::load_char__doc__[] =
-"load_char(charcode, flags=LOAD_DEFAULT)\n"
+"load_char(charcode, flags=LOAD_FORCE_AUTOHINT)\n"
 "\n"
 "Load character with charcode in current fontfile and set glyph.\n"
 "The flags argument can be a bitwise-or of the LOAD_XXX constants.\n"
@@ -813,7 +1075,7 @@ FT2Font::load_char(const Py::Tuple & args, const Py::Dict & kwargs) {
   //load a char using the unsigned long charcode
 
   args.verify_length(1);
-  long charcode = Py::Long(args[0]), flags = Py::Long(FT_LOAD_DEFAULT);
+  long charcode = Py::Long(args[0]), flags = Py::Long(FT_LOAD_FORCE_AUTOHINT);
   if (kwargs.hasKey("flags"))
     flags = Py::Long(kwargs["flags"]);
   
@@ -848,8 +1110,6 @@ FT2Font::get_width_height(const Py::Tuple & args) {
   _VERBOSE("FT2Font::get_width_height");
   args.verify_length(0);
 
-
-
   FT_BBox bbox = compute_string_bbox();
 
   Py::Tuple ret(2);
@@ -874,154 +1134,6 @@ FT2Font::get_descent(const Py::Tuple & args) {
   return Py::Int(- bbox.yMin);;
 }
 
-void
-FT2Font::draw_bitmap( FT_Bitmap*  bitmap,
-		      FT_Int      x,
-		      FT_Int      y) {
-  _VERBOSE("FT2Font::draw_bitmap");
-  FT_Int  i, j, p, q;
-  FT_Int  x_max = x + bitmap->width;
-  FT_Int  y_max = y + bitmap->rows;
-
-  FT_Int width = (FT_Int)image.width;
-  FT_Int height = (FT_Int)image.height;
-  for ( i = x, p = 0; i < x_max; i++, p++ )
-    {
-      for ( j = y, q = 0; j < y_max; j++, q++ )
-	{
-	  if ( i >= width || j >= height )
-	    continue;
-	  image.buffer[i + j*width] |= bitmap->buffer[p + q*bitmap->width];
-
-	}
-    }
-}
-
-char FT2Font::write_bitmap__doc__[] =
-"write_bitmap(fname)\n"
-"\n"
-"Write the bitmap to file fname\n"
-;
-Py::Object
-FT2Font::write_bitmap(const Py::Tuple & args) {
-  _VERBOSE("FT2Font::write_bitmap");
-
-  args.verify_length(1);
-
-  FT_Int  i, j;
-
-  std::string filename = Py::String(args[0]);
-
-  FILE *fh = fopen(filename.c_str(), "w");
-  FT_Int width = (FT_Int)image.width;
-  FT_Int height = (FT_Int)image.height;
-
-  for ( i = 0; i< height; i++)
-    for ( j = 0; j < width; ++j)
-      fputc(image.buffer[j + i*width], fh);
-
-  fclose(fh);
-
-  return Py::Object();
-}
-
-char FT2Font::draw_rect__doc__[] =
-"draw_rect(x0, y0, x1, y1)\n"
-"\n"
-"Draw a rect to the image.  It is your responsibility to set the dimensions\n"
-"of the image, eg, with set_bitmap_size\n"
-"\n"
-;
-Py::Object
-FT2Font::draw_rect(const Py::Tuple & args) {
-  _VERBOSE("FT2Font::draw_rect");
-
-  args.verify_length(4);
-
-  long x0 = Py::Int(args[0]);
-  long y0 = Py::Int(args[1]);
-  long x1 = Py::Int(args[2]);
-  long y1 = Py::Int(args[3]);
-
-  FT_Int iwidth = (FT_Int)image.width;
-  FT_Int iheight = (FT_Int)image.height;
-
-  if ( x0<0 || y0<0 || x1<0 || y1<0 ||
-       x0>iwidth || x1>iwidth ||
-       y0>iheight || y1>iheight )
-    throw Py::ValueError("Rect coords outside image bounds");
-
-  for (long i=x0; i<x1+1; ++i) {
-    image.buffer[i + y0*iwidth] = 255;
-    image.buffer[i + y1*iwidth] = 255;
-  }
-
-  for (long j=y0+1; j<y1; ++j) {
-    image.buffer[x0 + j*iwidth] = 255;
-    image.buffer[x1 + j*iwidth] = 255;
-  }
-  return Py::Object();
-}
-
-char FT2Font::draw_rect_filled__doc__[] =
-"draw_rect_filled(x0, y0, x1, y1)\n"
-"\n"
-"Draw a filled rect to the image.  It is your responsibility to set the\n"
-"dimensions of the image, eg, with set_bitmap_size\n"
-"\n"
-;
-Py::Object
-FT2Font::draw_rect_filled(const Py::Tuple & args) {
-  _VERBOSE("FT2Font::draw_rect_filled");
-
-  args.verify_length(4);
-
-  long x0 = Py::Int(args[0]);
-  long y0 = Py::Int(args[1]);
-  long x1 = Py::Int(args[2]);
-  long y1 = Py::Int(args[3]);
-
-  FT_Int iwidth = (FT_Int)image.width;
-  FT_Int iheight = (FT_Int)image.height;
-
-  if ( x0<0 || y0<0 || x1<0 || y1<0 ||
-       x0>iwidth || x1>iwidth ||
-       y0>iheight || y1>iheight )
-    throw Py::ValueError("Rect coords outside image bounds");
-
-  //~ for (long j=y0; j<y1; ++j) {
-      //~ for (long i=x0; i<x1+1; ++i) {
-        //~ image.buffer[i + j*iwidth] = 255;
-      //~ }
-  //~ }
-  for (long j=y0; j<y1+1; j++) {
-      for (long i=x0; i<x1+1; i++) {
-        image.buffer[i + j*iwidth] = 255;
-      }
-  }
-  return Py::Object();
-}
-
-char FT2Font::image_as_str__doc__[] =
-"width, height, s = image_as_str()\n"
-"\n"
-"Return the image buffer as a string\n"
-"\n"
-;
-Py::Object
-FT2Font::image_as_str(const Py::Tuple & args) {
-  _VERBOSE("FT2Font::image_as_str");
-  args.verify_length(0);
-
-  return Py::asObject(
-		      Py_BuildValue("lls#",
-				    image.width,
-				    image.height,
-				    image.buffer,
-				    image.width*image.height)
-		      );
-}
-
 char FT2Font::draw_glyphs_to_bitmap__doc__[] =
 "draw_glyphs_to_bitmap()\n"
 "\n"
@@ -1035,24 +1147,12 @@ FT2Font::draw_glyphs_to_bitmap(const Py::Tuple & args) {
   args.verify_length(0);
 
   FT_BBox string_bbox = compute_string_bbox();
+  size_t width = (string_bbox.xMax-string_bbox.xMin) / 64 + 2;
+  size_t height = (string_bbox.yMax-string_bbox.yMin) / 64 + 2;
 
-  image.width   = (string_bbox.xMax-string_bbox.xMin) / 64+2;
-  image.height  = (string_bbox.yMax-string_bbox.yMin) / 64+2;
-
-  image.offsetx = (int)(string_bbox.xMin/64.0);
-  if (angle==0)
-    image.offsety = -image.height;
-  else {
-    image.offsety = (int)(-string_bbox.yMax/64.0);
-  }
-
-  size_t numBytes = image.width*image.height;
-  delete [] image.buffer;
-  image.buffer = new unsigned char [numBytes];
-  for (size_t n=0; n<numBytes; n++)
-    image.buffer[n] = 0;
-
-
+  Py_XDECREF(image);
+  image = NULL;
+  image = new FT2Image(width, height);
 
   for ( size_t n = 0; n < glyphs.size(); n++ )
     {
@@ -1062,8 +1162,7 @@ FT2Font::draw_glyphs_to_bitmap(const Py::Tuple & args) {
       error = FT_Glyph_To_Bitmap(&glyphs[n],
 				 ft_render_mode_normal,
 				 0,
-				 //&pos[n],
-				 1  //destroy image;
+				 1
 				 );
       if (error)
 	throw Py::RuntimeError("Could not convert glyph to bitmap");
@@ -1072,13 +1171,10 @@ FT2Font::draw_glyphs_to_bitmap(const Py::Tuple & args) {
       // now, draw to our target surface (convert position)
 
       //bitmap left and top in pixel, string bbox in subpixel
-      FT_Int x = (FT_Int)(bitmap->left-string_bbox.xMin/64.);
-      FT_Int y = (FT_Int)(string_bbox.yMax/64.-bitmap->top+1);
-      //make sure the index is non-neg
-      x = x<0?0:x;
-      y = y<0?0:y;
+      FT_Int x = (FT_Int)(bitmap->left - (string_bbox.xMin / 64.));
+      FT_Int y = (FT_Int)((string_bbox.yMax / 64.) - bitmap->top + 1);
 
-      draw_bitmap( &bitmap->bitmap, x, y);
+      image->draw_bitmap( &bitmap->bitmap, x, y);
     }
 
   return Py::Object();
@@ -1108,8 +1204,7 @@ FT2Font::get_xys(const Py::Tuple & args) {
       error = FT_Glyph_To_Bitmap(&glyphs[n],
 				 ft_render_mode_normal,
 				 0,
-				 //&pos[n],
-				 1  //destroy image;
+				 1
 				 );
       if (error)
 	throw Py::RuntimeError("Could not convert glyph to bitmap");
@@ -1133,7 +1228,7 @@ FT2Font::get_xys(const Py::Tuple & args) {
 }
 
 char FT2Font::draw_glyph_to_bitmap__doc__[] =
-"draw_glyph_to_bitmap(x, y, glyph)\n"
+"draw_glyph_to_bitmap(bitmap, x, y, glyph)\n"
 "\n"
 "Draw a single glyph to the bitmap at pixel locations x,y\n"
 "Note it is your responsibility to set up the bitmap manually\n"
@@ -1147,16 +1242,17 @@ char FT2Font::draw_glyph_to_bitmap__doc__[] =
 Py::Object
 FT2Font::draw_glyph_to_bitmap(const Py::Tuple & args) {
   _VERBOSE("FT2Font::draw_glyph_to_bitmap");
-  args.verify_length(3);
+  args.verify_length(4);
 
-  if (image.width==0 || image.height==0)
-    throw Py::RuntimeError("You must first set the size of the bitmap with set_bitmap_size");
+  if (!FT2Image::check(args[0].ptr()))
+    throw Py::TypeError("Usage: draw_glyph_to_bitmap(bitmap, x,y,glyph)");
+  FT2Image* im = static_cast<FT2Image*>(args[0].ptr());
 
-  long x = Py::Int(args[0]);
-  long y = Py::Int(args[1]);
-  if (!Glyph::check(args[2].ptr()))
-    throw Py::TypeError("Usage: draw_glyph_to_bitmap(x,y,glyph)");
-  Glyph* glyph = static_cast<Glyph*>(args[2].ptr());
+  long x = Py::Int(args[1]);
+  long y = Py::Int(args[2]);
+  if (!Glyph::check(args[3].ptr()))
+    throw Py::TypeError("Usage: draw_glyph_to_bitmap(bitmap, x,y,glyph)");
+  Glyph* glyph = static_cast<Glyph*>(args[3].ptr());
 
   if ((size_t)glyph->glyphInd >= glyphs.size())
     throw Py::ValueError("glyph num is out of range");
@@ -1171,12 +1267,7 @@ FT2Font::draw_glyph_to_bitmap(const Py::Tuple & args) {
 
   FT_BitmapGlyph bitmap = (FT_BitmapGlyph)glyphs[glyph->glyphInd];
 
-  draw_bitmap( &bitmap->bitmap,
-	       //x + bitmap->left,
-	       x,
-	       //y+bitmap->top
-	       y
-	       );
+  im->draw_bitmap( &bitmap->bitmap, x + bitmap->left, y);
   return Py::Object();
 }
 
@@ -1343,11 +1434,11 @@ FT2Font::get_ps_font_info(const Py::Tuple & args)
   }
 
   Py::Tuple info(9);
-  info[0] = Py::String(fontinfo.version);
-  info[1] = Py::String(fontinfo.notice);
-  info[2] = Py::String(fontinfo.full_name);
-  info[3] = Py::String(fontinfo.family_name);
-  info[4] = Py::String(fontinfo.weight);
+  info[0] = Py::String(fontinfo.version ? fontinfo.version : "");
+  info[1] = Py::String(fontinfo.notice ? fontinfo.notice : "");
+  info[2] = Py::String(fontinfo.full_name ? fontinfo.full_name : "");
+  info[3] = Py::String(fontinfo.family_name ? fontinfo.family_name : "");
+  info[4] = Py::String(fontinfo.weight ? fontinfo.weight : "");
   info[5] = Py::Long(fontinfo.italic_angle);
   info[6] = Py::Int(fontinfo.is_fixed_pitch);
   info[7] = Py::Int(fontinfo.underline_position);
@@ -1368,8 +1459,8 @@ FT2Font::get_sfnt_table(const Py::Tuple & args) {
   std::string tagname = Py::String(args[0]);
 
   int tag;
-  char *tags[] = {"head", "maxp", "OS/2", "hhea",
-		  "vhea", "post", "pclt",  NULL};
+  const char *tags[] = {"head", "maxp", "OS/2", "hhea",
+			"vhea", "post", "pclt",  NULL};
 
   for (tag=0; tags[tag] != NULL; tag++)
     if (strcmp(tagname.c_str(), tags[tag]) == 0)
@@ -1568,6 +1659,53 @@ FT2Font::get_sfnt_table(const Py::Tuple & args) {
   }
 }
 
+char FT2Font::get_image__doc__ [] = 
+  "get_image()\n"
+  "\n"
+  "Returns the underlying image buffer for this font object.\n";
+Py::Object
+FT2Font::get_image (const Py::Tuple &args) {
+  args.verify_length(0);
+  if (image) {
+    Py_XINCREF(image);
+    return Py::asObject(image);
+  } 
+  throw Py::RuntimeError("You must call .set_text() before .get_image()");
+}
+
+char FT2Font::attach_file__doc__ [] =
+  "attach_file(filename)\n"
+  "\n"
+  "Attach a file with extra information on the font\n"
+  "(in practice, an AFM file with the metrics of a Type 1 font).\n"
+  "Throws an exception if unsuccessful.\n";
+Py::Object
+FT2Font::attach_file (const Py::Tuple &args) {
+  args.verify_length(1);
+
+  std::string filename = Py::String(args[0]);
+  FT_Error error = 
+    FT_Attach_File(face, filename.c_str());
+
+  if (error) {
+    std::ostringstream s;
+    s << "Could not attach file " << filename
+      << " (freetype error code " << error << ")" << std::endl;
+    throw Py::RuntimeError(s.str());
+  }
+  return Py::Object();
+}
+
+Py::Object
+ft2font_module::new_ft2image (const Py::Tuple &args) {
+  args.verify_length(2);
+
+  int width = Py::Int(args[0]);
+  int height = Py::Int(args[1]);
+
+  return Py::asObject( new FT2Image(width, height) );
+}
+
 Py::Object
 ft2font_module::new_ft2font (const Py::Tuple &args) {
   _VERBOSE("ft2font_module::new_ft2font ");
@@ -1577,6 +1715,30 @@ ft2font_module::new_ft2font (const Py::Tuple &args) {
   return Py::asObject( new FT2Font(facefile) );
 }
 
+void
+FT2Image::init_type() {
+ _VERBOSE("FT2Image::init_type");
+ behaviors().name("FT2Image");
+ behaviors().doc("FT2Image");
+
+ add_varargs_method("write_bitmap", &FT2Image::py_write_bitmap,
+		    FT2Image::write_bitmap__doc__);
+ add_varargs_method("draw_rect", &FT2Image::py_draw_rect,
+		    FT2Image::draw_rect__doc__);
+ add_varargs_method("draw_rect_filled", &FT2Image::py_draw_rect_filled,
+		    FT2Image::draw_rect_filled__doc__);
+ add_varargs_method("as_str", &FT2Image::py_as_str,
+		    FT2Image::as_str__doc__);
+ add_varargs_method("as_rgb_str", &FT2Image::py_as_rgb_str,
+		    FT2Image::as_rgb_str__doc__);
+ add_varargs_method("as_rgba_str", &FT2Image::py_as_rgba_str,
+		    FT2Image::as_rgba_str__doc__);
+ add_varargs_method("get_width", &FT2Image::py_get_width,
+		    "Returns the width of the image");
+ add_varargs_method("get_height", &FT2Image::py_get_height,
+		    "Returns the height of the image");
+}
+
 void
 Glyph::init_type() {
   _VERBOSE("Glyph::init_type");
@@ -1584,9 +1746,8 @@ Glyph::init_type() {
   behaviors().doc("Glyph");
   behaviors().supportGetattr();
   behaviors().supportSetattr();
-
-
 }
+
 void
 FT2Font::init_type() {
   _VERBOSE("FT2Font::init_type");
@@ -1595,14 +1756,6 @@ FT2Font::init_type() {
 
   add_varargs_method("clear", &FT2Font::clear,
 		     FT2Font::clear__doc__);
-  add_varargs_method("write_bitmap", &FT2Font::write_bitmap,
-		     FT2Font::write_bitmap__doc__);
-  add_varargs_method("set_bitmap_size", &FT2Font::set_bitmap_size,
-		     FT2Font::load_char__doc__);
-  add_varargs_method("draw_rect",&FT2Font::draw_rect,
-		     FT2Font::draw_rect__doc__);
-    add_varargs_method("draw_rect_filled",&FT2Font::draw_rect_filled,
-		     FT2Font::draw_rect_filled__doc__);
   add_varargs_method("draw_glyph_to_bitmap", &FT2Font::draw_glyph_to_bitmap,
 		     FT2Font::draw_glyph_to_bitmap__doc__);
   add_varargs_method("draw_glyphs_to_bitmap", &FT2Font::draw_glyphs_to_bitmap,
@@ -1614,11 +1767,9 @@ FT2Font::init_type() {
 		     FT2Font::get_glyph__doc__);
   add_varargs_method("get_num_glyphs", &FT2Font::get_num_glyphs,
 		     FT2Font::get_num_glyphs__doc__);
-  add_varargs_method("image_as_str", &FT2Font::image_as_str,
-		     FT2Font::image_as_str__doc__);
   add_keyword_method("load_char", &FT2Font::load_char,
 		     FT2Font::load_char__doc__);
-  add_varargs_method("set_text", &FT2Font::set_text,
+  add_keyword_method("set_text", &FT2Font::set_text,
 		     FT2Font::set_text__doc__);
   add_varargs_method("set_size", &FT2Font::set_size,
 		     FT2Font::set_size__doc__);
@@ -1643,9 +1794,10 @@ FT2Font::init_type() {
 		     FT2Font::get_ps_font_info__doc__);
   add_varargs_method("get_sfnt_table", &FT2Font::get_sfnt_table,
 		     FT2Font::get_sfnt_table__doc__);
-  add_varargs_method("horiz_image_to_vert_image",
-		     &FT2Font::horiz_image_to_vert_image,
-		     FT2Font::horiz_image_to_vert_image__doc__);
+  add_varargs_method("get_image", &FT2Font::get_image,
+		     FT2Font::get_image__doc__);
+  add_varargs_method("attach_file", &FT2Font::attach_file,
+                     FT2Font::attach_file__doc__);
 
   behaviors().supportGetattr();
   behaviors().supportSetattr();
@@ -1701,6 +1853,7 @@ char ft2font_new__doc__[] =
 "  max_advance_height     same for vertical layout\n"
 "  underline_position     vertical position of the underline bar\n"
 "  underline_thickness    vertical thickness of the underline\n"
+"  postscript_name        PostScript name of the font\n"
 ;
 
 #if defined(_MSC_VER)
@@ -1764,6 +1917,5 @@ initft2font(void)
 }
 
 ft2font_module::~ft2font_module() {
-
   FT_Done_FreeType( _ft2Library );
 }
